<?php
// $Id: EditHelpdeskTicketResponse.php,v 1.18 2006/06/27 16:32:35 bsmith Exp $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This action takes the information provided by the EditHelpdeskTicket action, and applies it to the database,
 * thus editing a helpdesk ticket in the system. If the provided helpdesk ticket ID does not exist, an error is
 * thrown. Provided no errors occur, once the helpdesk ticket is edited, this action redirects to the
 * ShowHelpdeskTicketDetails action, where the new changes can be viewed.
 *
 * @author  Vic Fryzel
 */
class EditHelpdeskTicketResponse extends MB_Action {
	/**
	 * The constructor for an EditHelpdeskTicketResponse action object. This calls the parent constructor,
	 * sets the string representation of this object and sets up $this->params.
	 */
	function EditHelpdeskTicketResponse() {
		parent::MB_Action();
		$this->stringRep = "EditHelpdeskTicketResponse";
		$this->params = array("hdTicketID", "hdTicketSubject", "hdTicketBody",
			"hdTicketDateCreated", "hdTicketDateModified", "hdTicketPriorityID",
			"hdTicketStatusID");
	}

	/**
	 * The main method of this class that executes the EditHelpdeskTicketResponse action, and returns
	 * success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$ticketID = ((int) $_REQUEST["hdTicketID"]);
		if(!$ticketID) {
			$this->addError(TRANS_ACTIONERROR, "No helpdesk ticket ID given.");
		} else {
			/**
			 *	Sub command:: assignClient
			 *	Assign a client to the ticket
			 */
			if (notNull( $_REQUEST["a"] ) && ($_REQUEST["a"] == "assignClient")) {
				if (!notZero( $_REQUEST["clientContactID"] )) {
					$this->addError(TRANS_ACTIONERROR, "Client Contact ID is not given.");
				} else {
					// first make sure client exists
					$query =& new MBQuery("GetClients", "", array("getContactData" => 4, "clientContactID" => (int)$_REQUEST["clientContactID"]));
					$client = $query->getResult();

					if (!$client->getErrorCount()) {
						$client = grabSimpleMBAPIResults($client->getStructure());

						if (notZero( $client[0]["contacts"][0]["contact"][0]["clientContactID"] )) {
							/// YES!
							$query =& new MBQuery("SetHelpdeskTickets", "update", array(
														"hdTicketID" => $ticketID,
														"hdTicketDateModified" => mktime(),
														"userID" => (int)$_REQUEST["clientContactID"],
														"userType" => 1)); // regular user type
							$result = $query->getResult();

							if ($result->getErrorCount()) {
								$this->addError(TRANS_ACTIONERROR, $result->getLastError());
							}
						}
					} else {
						$this->addError(TRANS_ACTIONERROR, "Couldn't Find Client.");
					}
				}
				
				if(!$this->getNumErrors()) {
					redirect("helpdesk.php", "ShowHelpdeskTicketDetails", array("hdTicketID" => $ticketID));
				}

				$this->addErrorTemplates();
				return $this->hasExecutedSuccessfully();
			}
			
			$ticket = $this->getMBAPIData("GetHelpdeskTickets", "helpdeskTickets", "helpdeskTicket",array(
			 						"hdTicketID" => $ticketID,
			 						"getHelpdeskDepartmentData" => 1,
									"getHelpdeskBucketData" => 1));
			$ticket = $ticket[0][0];
			
			$data = array();
				
			//get rid of duplicates
			if (is_array( $_REQUEST["hdBucketIDs"] ))
				$_REQUEST["hdBucketIDs"] = array_flip(array_flip($_REQUEST["hdBucketIDs"]));

			if(count($_REQUEST["hdDepartmentIDs"])) {
				$depts = array();
				if(is_array($ticket["hdDepartmentIDs"][0]["hdDepartmentID"])) {
					$depts = $ticket["hdDepartmentIDs"][0]["hdDepartmentID"];
				}
				$data["hdDepartmentIDs"] = getAddIDXML("hdDepartmentID", $depts, $_REQUEST["hdDepartmentIDs"]);
				$data["removeHDDepartmentIDs"] = getRemoveIDXML("hdDepartmentID", $depts, array());
				#$data["removeHDDepartmentIDs"] = getRemoveIDXML("hdDepartmentID", $depts, $_REQUEST["hdDepartmentIDs"]);
			}
			if(count($_REQUEST["hdBucketIDs"])) {
				$buckets = array();
				if(is_array($ticket["hdBucketIDs"][0]["hdBucketID"])) {
					$buckets = $ticket["hdBucketIDs"][0]["hdBucketID"];
				}
				
				$data["hdBucketIDs"] = getAddIDXML("hdBucketID", $buckets, $_REQUEST["hdBucketIDs"]);
				$data["removeHDBucketIDs"] = getRemoveIDXML("hdBucketID", $buckets, array());
				#$data["removeHDBucketIDs"] = getRemoveIDXML("hdBucketID", $buckets, $_REQUEST["hdBucketIDs"]);
			}
			
			if($_REQUEST["hdTicketResponseBody"]) {
				$data["helpdeskTicketResponses"] = "<helpdeskTicketResponse>\n"
					."<userID>".$CONF["user"]["id"]."</userID>\n"
					."<hdTicketRespBody>".$_REQUEST["hdTicketResponseBody"]."</hdTicketRespBody>\n"
					."<hdTicketRespDateCreated>".mktime()."</hdTicketRespDateCreated>\n"
					."<hdTicketRespDateModified>".mktime()."</hdTicketRespDateModified>\n"
					."</helpdeskTicketResponse>\n";
			}

			if($_REQUEST["hdTicketNoteBody"]) {
				$data["helpdeskTicketNotes"] = "<helpdeskTicketNote>\n"
					."<adminID>".$CONF["user"]["id"]."</adminID>\n"
					."<hdTicketNoteBody>".$_REQUEST["hdTicketNoteBody"]."</hdTicketNoteBody>\n"
					."<hdTicketNoteDateCreated>".mktime()."</hdTicketNoteDateCreated>\n"
					."<hdTicketNoteDateModified>".mktime()."</hdTicketNoteDateModified>\n"
					."</helpdeskTicketNote>\n";
			}

			if (!notZero( $data["hdTicketDateModified"] )) {
				$data["hdTicketDateModified"] = mktime();
			}

			dbg("#### Date sent to SetHelpdeskTickets ####");
			dbg($data);
			
			// Generate the query
			$query = $this->getMBAPIQuery("SetHelpdeskTickets", "update", $data);
			$data = dispatchMBAPI($query, 1);
			
			dbg("#### Recieved from SetHelpdeskTickets ####");
			dbg($data);
			
			if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
				redirect("helpdesk.php", "ShowHelpdeskTicketDetails", array("hdTicketID" => $ticketID));
			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // EditHelpdeskTicketResponse
?>