<?php
// $Id: ShowBalanceSheet.php,v 1.6 2006/06/01 01:14:58 kbailey Exp $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");
require_once mb_path("/lib-modernbill/include/accounting/Accounting.php");

/**
 * This class is a child class of MB_Action. It displays the Balance Sheet
 *
 * @author  Kris Bailey
 */
class ShowBalanceSheet extends MB_Action {

	function ShowBalanceSheet() {
		parent::MB_Action();

		$this->stringRep = "ShowBalanceSheet";

		list($m, $d, $y) = explode("/", $_REQUEST["dateQuery"][0]);

		$this->endTime = @mktime(0,0,0,$m,$d+1,$y)-1;

		unset($_REQUEST["dateQuery"][0]);

	}

	function execute() {
		global $CONF;
		parent::execute();

        $this->smartyVars = array();
        $this->smartyVars["accountTypes"] = $CONF["ledgerCategories"];

        $this->smartyVars["endDate"] = $this->endTime;

        $accountsXML = "
                    <mbapi>
                       <command>GetAccountingLedgerAccounts</command>
                       <showXMLHeader>0</showXMLHeader>
                       <params>
                         <sortDir>ASC</sortDir>
                         <sortColumn>ledgerAccountNumber</sortColumn>
                         <recordOffset>0</recordOffset>
                         <numRecords>-1</numRecords>
                       </params>
                    </mbapi>
                    ";
        $accounts = dispatchMBAPI($accountsXML, 1);

        $this->smartyVars["ledgerAccounts"] = array();
        if (isset($accounts["mbapi"][0]["results"][0]["ledger_accounts"][0]["account"])){
            foreach ($accounts["mbapi"][0]["results"][0]["ledger_accounts"][0]["account"] as $ledgerAccount){
                $this->smartyVars["ledgerAccounts"][$ledgerAccount["ledgerAccountID"][0]] = collapseArrayElements($ledgerAccount);
            }
        }

        $ledgerEntriesXML = "
                    <mbapi>
                      <command>GetAccountingGeneralLedger</command>
                      <showXMLHeader>0</showXMLHeader>
                      <params>
                        <generalLedgerDateCreatedGreatest>".$this->endTime."</generalLedgerDateCreatedGreatest>
                        <groupByAndSum>1</groupByAndSum>
                        <sortDir>ASC</sortDir>
                        <sortColumn>generalLedgerID</sortColumn>
                        <recordOffset>0</recordOffset>
                        <numRecords>-1</numRecords>
                      </params>
                    </mbapi>
                    ";
        $ledgerEntries = dispatchMBAPI($ledgerEntriesXML, 1);

        $this->smartyVars["accountTotals"] = array();

        if (isset($ledgerEntries["mbapi"][0]["results"][0]["accounting_general_ledger"][0]["ledger"])){
            foreach ($ledgerEntries["mbapi"][0]["results"][0]["accounting_general_ledger"][0]["ledger"] as $entry){
                $this->smartyVars["accountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["accountInfo"] = collapseArrayElements($entry);
                $this->smartyVars["accountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["credits"] += $entry["generalLedgerCreditAmount"][0];
                $this->smartyVars["accountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["debits"] += $entry["generalLedgerDebitAmount"][0];
            }
        }

        $this->smartyVars["ledgerTypeTotals"] = array();
        $this->smartyVars["ledgerAccountsTotals"] = array();
        foreach ($this->smartyVars["accountTotals"] as $currencyID => $accountTotals){
            $this->smartyVars["ledgerTypeTotals"][$currencyID] = array();
            foreach ($accountTotals as $accountID => $totalInfo){
                $accountType = $CONF["ledgerCategories"][$totalInfo["accountInfo"]["ledgerAccountType"]][0];

                switch ($accountType){
                    case "asset":
                        $totalInfo["total"] = $totalInfo["debits"]-$totalInfo["credits"];
                        $this->smartyVars["ledgerAccountsTotals"][$currencyID]["asset"][] = $totalInfo;
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["asset"] -= $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["asset"] += $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] -= $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] += $totalInfo["debits"];
                        break;
                    case "liability":
                        $totalInfo["total"] = $totalInfo["credits"]-$totalInfo["debits"];
                        $this->smartyVars["ledgerAccountsTotals"][$currencyID]["liability"][] = $totalInfo;
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["liability"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["liability"] -= $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] -= $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] += $totalInfo["debits"];
                        break;
                    case "equity":
                        $totalInfo["total"] = $totalInfo["credits"]-$totalInfo["debits"];
                        $this->smartyVars["ledgerAccountsTotals"][$currencyID]["equity"][] = $totalInfo;
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] -= $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] -= $totalInfo["debits"];
                        break;

////////////////////////////////
                    case "revenue":
                    case "otherRev":
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] -= $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] -= $totalInfo["debits"];
                        break;
                    case "cogs":

                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] -= $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] -= $totalInfo["debits"];
                        break;
                    case "expense":
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] -= $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] -= $totalInfo["debits"];
                        break;

////////////////////////////////

                }

            }
        }

        ksort($this->smartyVars["ledgerTypeTotals"]);

		$this->addTemplate("accounting/ShowBalanceSheet", $this->smartyVars);

	} // execute
} // MB_Action
?>
