<?php
// $Id: ShowCerts.php,v 1.18 2006/07/21 19:51:01 bsmith Exp $
// +------------------------------------------------------------------------------+
// | MBAPI [TM] .:. Client Billing Engine                                         |
// +------------------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                                  |
// +------------------------------------------------------------------------------+
// | This source file is subject to the ModernGigabyte End User License Agreement |
// | (EULA), that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at http://www.mbapi.com/eula.php     |
// |                                                                              |
// | If you did not receive a copy of the MBAPI license and are unable to obtain  |
// | it through the world-wide-web, please send a note to license@mbapi.com so we |
// | can email you a copy immediately.                                            |
// +------------------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>                       |
// | Support: http://www.modernsupport.com/                                       |
// +------------------------------------------------------------------------------+
// | ModernGigabyte, ModernBill, and MBAPI are trademarks of ModernGigabyte, LLC. |
// +------------------------------------------------------------------------------+

require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");
require_once mb_path("/lib-modernbill/include/config/internal/certs.php");
require_once mb_path("/lib-modernbill/include/config/internal/cycles.php");

/**
 * This Action object collects information on all certs that match the given parameters, and 
 * generates the display of that information.
 *
 * @author  Michael Fountain
 */
class ShowCerts extends MB_Action {
	/**
	 * The constructor for a ShowCerts action object. This merely calls the parent constructor, 
	 * and sets the string representation of this object. This method also sets up $this->params and 
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function ShowCerts() {
		global $CONF;
		parent::MB_Action();
		$this->stringRep = "ShowCerts";
		
		// define all possible params that we will accept
		$this->params = array
		(
			"certID",
			"packageID",
			"clientID",
			"orID",
			"rID",
			"domainName",
			"ODServerCount",

		);

		// if we are searching for any possible match, set VALID params now
		$validCompares = array("Least","Greatest");
		if ($_REQUEST["amountQuery"]) {
			// loop through each amount entered
			foreach($_REQUEST["amountQuery"] as $key => $amount) {
				// make sure the amount is not empty
				if ($amount!="" && ($_REQUEST["amountCompare"][$key]=="" || in_array($_REQUEST["amountCompare"][$key],$validCompares))) {
					$_REQUEST[$_REQUEST["amountColumn"].$_REQUEST["amountCompare"][$key]] = $amount;
				}
			}
		}
		if ($_REQUEST["dateQuery"]) {
			// loop through each date entered
			foreach($_REQUEST["dateQuery"] as $key => $date) {
				// make sure the date is not empty
				if ($date!="" && ($_REQUEST["dateCompare"][$key]=="" || in_array($_REQUEST["dateCompare"][$key],$validCompares))) {
					$_REQUEST[$_REQUEST["dateColumn"].$_REQUEST["dateCompare"][$key]] = $date;
				}
			}
		}
		if ($_REQUEST["query"]!="") {
			// test if the searching for a specific column
			if ($_REQUEST["queryColumn"]!="") {
				$_REQUEST[$_REQUEST["queryColumn"]] = $_REQUEST["query"];
			} else {
				// test if query is an int, if yes, assume we are searching for the ID
				if (((int) $_REQUEST["query"])) {
					$_REQUEST["certID"] = $_REQUEST["query"];
				} else {
					// if not an int, then search all other non-ID columns
					foreach($this->params as $value) {
						if ($value!="certID") {
							$_REQUEST[$value] = $_REQUEST["query"];
						}
					}
				}
			}
		}

		// add filtered columns to $this->params LAST, they are NOT searchable
		$filteredColumns = array
		(
			"ODProductCode",
			"ODValidityPeriod",
			"orStatus"
		);
		foreach($filteredColumns as $filteredColumn) {
			if (is_numeric($_REQUEST[$filteredColumn])) {
				$this->params[] = $filteredColumn;
			}
		}

		$this->sortColumns = array
		(
			"certID",
			"packageID",
			"clientID",
			"domainName",
		);

	}

	/**
	 * The main method of this class that executes the ShowCerts action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$query = $this->getMBAPIQuery("GetCerts", "", array("getProductPackage" => 1, "getClientData"=>1));
		$data = dispatchMBAPI($query, 1);
		
		dbg($data);
		
		$results = grabSimpleMBAPIResults($data);
		if($this->addMBAPIErrors($data) === 0) {

			for( $i=0, $n=count( $results ); $i<$n; $i++ ) {
				if (notZero( $results[ $i ]['clientID'] )) {
					$results[ $i ] = array_merge($results[ $i ], oneOf('clientWithPrimary', array('clientID' => $results[ $i ]['clientID'])));
				}
			}
			$this->smartyVars["certs"] = $results;

			$this->smartyVars["paginationLinks"] = $data["mbapi"][0]["results"][0]["paginationLinks"][0];
			$this->smartyVars["packageStatuses"] = getStatuses("package");
			$this->mergeCycleData();
			
			// assign filtered columns
			$filteredColumns = array
			(
				"ODProductCode",
				"ODValidityPeriod",
				"orStatus"
			);
			foreach($filteredColumns as $filteredColumn) {
				if (is_numeric($_REQUEST[$filteredColumn])) {
					$smartyVars[$filteredColumn] = ((int) $_REQUEST[$filteredColumn]);
					$smartyVars["filters"][] = $filteredColumn;
				}
			}

			
			$this->addTemplate("certs/ShowCerts");
			
			
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // ShowCerts
?>