<?php
// $Id: ShowPackages.php,v 1.28 2006/07/26 14:00:49 mgarrett Exp $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");
require_once mb_path("/lib-modernbill/include/config/internal/cycles.php");

/**
 * This Action object collects information on all packages that match the given parameters, and
 * generates the display of that information.
 *
 * @author  Michael Fountain
 */
class ShowPackages extends MB_Action {
	/**
	 * The constructor for a ShowPackages action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. This method also sets up $this->params and
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function ShowPackages() {
		global $CONF;
		parent::MB_Action();
		$this->stringRep = "ShowPackages";

		// define all possible params that we will accept
		$this->params = array
		(
			"packageID",
			"productID",
			"clientID",

			"packageName",
			"packageQuantity",
			"packageDiscount",
			"packageStatus",
			"packageBillingCycle",
			"packageComments",

			// packagePrice options
			"packagePrice",
			"packagePriceLeast",
			"packagePriceGreatest",

			// packageSetup options
			"packageSetup",
			"packageSetupLeast",
			"packageSetupGreatest",

			// packageDateBeginRenewal options
			"packageDateBeginRenewal",
			"packageDateBeginRenewalLeast",
			"packageDateBeginRenewalGreatest",

			// packageDateNextRenewal options
			"packageDateNextRenewal",
			"packageDateNextRenewalLeast",
			"packageDateNextRenewalGreatest",

			// packageDateLastRenewal options
			"packageDateLastRenewal",
			"packageDateLastRenewalLeast",
			"packageDateLastRenewalGreatest",

		);

		// if we are searching for any possible match, set VALID params now
		$validCompares = array("Least","Greatest");
		if ($_REQUEST["amountQuery"]) {
			// loop through each amount entered
			foreach($_REQUEST["amountQuery"] as $key => $amount) {
				// make sure the amount is not empty
				if ($amount!="" && ($_REQUEST["amountCompare"][$key]=="" || in_array($_REQUEST["amountCompare"][$key],$validCompares))) {
					$_REQUEST[$_REQUEST["amountColumn"].$_REQUEST["amountCompare"][$key]] = $amount;
				}
			}
		}
		if ($_REQUEST["dateQuery"]) {
			// loop through each date entered
			foreach($_REQUEST["dateQuery"] as $key => $date) {
				// make sure the date is not empty
				if ($date!="" && ($_REQUEST["dateCompare"][$key]=="" || in_array($_REQUEST["dateCompare"][$key],$validCompares))) {
					$_REQUEST[$_REQUEST["dateColumn"].$_REQUEST["dateCompare"][$key]] = $date;
				}
			}
		}
		if ($_REQUEST["query"]!="") {
			// test if the searching for a specific column
			if ($_REQUEST["queryColumn"]!="") {
				$_REQUEST[$_REQUEST["queryColumn"]] = $_REQUEST["query"];
			} else {
				// test if query is an int, if yes, assume we are searching for the ID
				if (((int) $_REQUEST["query"])) {
					$_REQUEST["packageID"] = $_REQUEST["query"];
				} else {
					// if not an int, then search all other non-ID columns
					foreach($this->params as $value) {
						if ($value!="packageID") {
							$_REQUEST[secureInput($value)] = $_REQUEST["query"];
						}
					}
				}
			}
		}

		// add filtered columns to $this->params LAST, they are NOT searchable
		$filteredColumns = array
		(
			"packageStatus",
			"cycleID"
		);
		foreach($filteredColumns as $filteredColumn) {
			if (is_numeric($_REQUEST[$filteredColumn])) {
				$this->params[] = $filteredColumn;
			}
		}

		$this->sortColumns = array
		(
			"packageID",
			"productID",
			"packageName",
			"packageQuantity",
			"packageDiscount",
			"packageBillingCycle",
			"packageStatus",
			"packageComments",
			"packagePrice",
			"packageSetup",
			"packageDateBeginRenewal",
			"packageDateNextRenewal",
			"packageDateLastRenewal",
			"clientLastName"
		);

	}

	/**
	 * Execute this action object, and display any output generated to the screen.
	 *
	 * @return  1 if no errors occurred during execution, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$query = $this->getMBAPIQuery("GetPackages", "", array("getClientData"=>1,"getDomainData"=>1), 1, 1, 1, 1);
		$data = dispatchMBAPI($query, 1);
		$results =& $data["mbapi"][0]["results"][0];
		if($this->addMBAPIErrors($data) === 0) {

			$this->smartyVars["packages"] = $results["packages"][0]["package"];
			$this->smartyVars["paginationLinks"] = collapseArrayElements($data["mbapi"][0]["results"][0]["paginationLinks"][0]);
			if($data["mbapi"][0]["header"][0]["numResults"][0] == 1) {
				$this->smartyVars["packages"] = array(0 => $results["packages"][0]["package"]);
			}

			//Pull the event queue items out
			$queueStatuses = getStatuses("queue");
			$queueItems = array();

			$packagesTemp = correctArray($results["packages"][0]["package"]);

			if($packagesTemp) {
				foreach($packagesTemp as $key => $value) {
					if((int) $value["packageID"]) {
						$query = $this->getMBAPIQuery("GetSystemQueue", "", array("packageID"=>$value["packageID"]));
						$data = dispatchMBAPI($query, 1);


						$results =& $data["mbapi"][0]["results"][0]["systemQueue"][0]["systemQueueEvent"];
						if($results) {
							foreach($results as $v) {
								$this->smartyVars["packages"][0][$key]["systemQueueItems"][$v["systemQueueID"][0]] =
														array("configGroupID" => $v["configGroupID"][0],
																		"packageID" => $v["packageID"][0],
																		"serverGroupID" => $v["serverGroupID"][0],
																		"systemQueueStatus" => $queueStatuses[(int)$v["systemQueueStatus"][0]],
																		"systemQueueCommand" => $v["systemQueueCommand"][0],
																		"systemQueueDateFinished" => $v["systemQueueDateFinished"][0],
																			);
							}
						}

					}
				}
			}
			$this->smartyVars["packageStatuses"] = getStatuses("package");
			$this->smartyVars["cycles"] = getCyclesArray();

			// assign filtered columns
			$filteredColumns = array
			(
				"packageStatus",
				"cycleID"
			);
			foreach($filteredColumns as $filteredColumn) {
				if (is_numeric($_REQUEST[$filteredColumn])) {
					$this->smartyVars[$filteredColumn] = ((int) $_REQUEST[$filteredColumn]);
					$this->smartyVars["filters"][] = $filteredColumn;
				}
			}



			// assign sortable columns for advanced options
			$this->smartyVars["sortColumns"] = array
			(
				"packageID" => TRANS_PACKAGE_ID,
				"clientLastName" => TRANS_LAST_NAME,
				"packageName" => TRANS_PACKAGE,
				"packagePrice" => TRANS_PRICE,
				"packageDateNextRenewal" => TRANS_DATE_NEXT_RENEWAL,
			);

			$this->smartyVars = correctArray($this->smartyVars);

			$this->addTemplate("packages/ShowPackages", $this->smartyVars);
			
			
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // ShowPackages
?>
