<?php
// $Id: ShowDashboard.php,v 1.12 2006/06/24 16:43:30 bsmith Exp $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+

require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");
require_once mb_path("/lib-modernbill/include/config/internal/cycles.php");

/**
 * This Action object displays a client dashboard.
 *
 * @author  Michael Fountain
 */
class ShowDashboard extends MB_Action {
	/**
	 * The constructor for a ShowDashboard action object. This method calls the parent constructor,
	 * and sets the string representation of this object.
	 */
	function ShowDashboard() {
		parent::MB_Action();
		$this->stringRep = "ShowDashboard";
	}

	/**
	 * The main method of this class that executes the ShowDashboard action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

        	// get client data
		$clientID = ((int) $CONF["user"]["id"]);
		if(!$clientID) {
			$this->addError(TRANS_ACTIONERROR, "No client ID given.");
		} else {
	
			// ---------------------------------------------------
			// Get client data ...
			// ---------------------------------------------------
			$this->mergeClientData();
		
			// ---------------------------------------------------
			// Get DUE invoices ...
			// ---------------------------------------------------
			$this->smartyVars["invoices"] = array(); // init
			$params = array(
				"clientID" => $clientID,
				"getDueInovicesOnly" => 1,
				"getInvoiceLineItemsData" => 1);
			$query = getMBAPIQuery("GetInvoices", "",  $params, "ASC", "invoiceID", 0, 5);
			$data = dispatchMBAPI($query, 1);
	
			$results =& $data["mbapi"][0]["results"][0];
			if($this->addMBAPIErrors($data) === 0) {
				// set master invoices data array
				$this->smartyVars["invoices"] = collapseArrayElements(protectArray($results["invoices"][0]["invoice"]));
				// build invoice line item data
				if($this->smartyVars["invoices"]) {
					for ($i = 0; $i < count($this->smartyVars["invoices"]); $i++){
						if (!$this->smartyVars["invoices"][$i]["lineItems"]["lineItem"][0]["clientID"]){
							$this->smartyVars["invoices"][$i]["lineItems"]["lineItem"] = array($this->smartyVars["invoices"][$i]["lineItems"]["lineItem"]);
						}
					}
				}
	
				// set master invoices line item data array
				$lineItems = array();
				$c = count($results["invoices"][0]["invoice"]);
				for($i = 0; $i < $c; $i++) {
					// leave this uncollapsed so that the template code can work with both singular and plural results;
					// see the comment in the template for details
					$lineItemsData =& $results["invoices"][0]["invoice"][$i]["lineItems"][0]["lineItem"];
					$lineItems[$results["invoices"][0]["invoice"][$i]["invoiceID"][0]] = collapseArrayElements(protectArray($lineItemsData));
				}
				$this->smartyVars["lineItems"] = $lineItems;
			}
	
			// ---------------------------------------------------
			// Get ACTIVE packages ...
			// ---------------------------------------------------
			$this->smartyVars["packages"] = array(); // init
			$this->smartyVars["packageStatuses"] = getStatuses("package");
			$this->smartyVars["cycles"] = getCyclesArray();
			$params = array (
				"clientID" => $clientID,
				"getBillablePackagesOnly" => 1);
			$query = getMBAPIQuery("GetPackages", "",  $params, "ASC", "packageStatus", 0, 5);
			$data = dispatchMBAPI($query, 1);
			$results =& $data["mbapi"][0]["results"][0];
			if($this->addMBAPIErrors($data) === 0) {
				$this->smartyVars["packages"] = collapseArrayElements(protectArray($results["packages"][0]["package"]));
			}
	
			// ---------------------------------------------------
			// Get PPC Offers ...
			// ---------------------------------------------------
			loadConfigs("other");
			if ($CONF["other"]["ppc"]["ppcfactory"]["enablePPCfactory"]) {
				$params = array ("codeClientID" => $clientID);
				$query = getMBAPIQuery("GetPPCCodes", "",  $params, "ASC", "", 0, 1);
				$data = dispatchMBAPI($query, 1);
				if($this->addMBAPIErrors($data) === 0) {
					$data = correctArray($data);
					dbg($data);
					switch ($data["mbapi"][0]["results"][0]["ppcCodes"][0]["codeStatus"]) {
					// offer has been accepted, now we need to activate it
					case getAPIStatusID("ppcCodes", "accepted"):
						$ppcArgs = array(
							"customerKey" => $data["mbapi"][0]["results"][0]["ppcCodes"][0]["codeClientID"],
							"codeID" => $data["mbapi"][0]["results"][0]["ppcCodes"][0]["codeRemoteID"],
							"apiAction" => "activate");
						$ppcQuery = getMBAPIQuery("ProcessPPCfactory", "", $ppcArgs);
						$ppcData = correctArray(dispatchMBAPI($ppcQuery, 1));
						dbg($ppcData);
						/* DEBUG
						echo "<pre>".print_r($ppcArgs,1)."</pre>";
						echo "<pre>".print_r($ppcData,1)."</pre>";
						/**/
			
						if ($ppcData["mbapi"][0]["header"][0]["success"]) {
							$data["mbapi"][0]["results"][0]["ppcCodes"][0]["codeToUse"] = $ppcData["mbapi"][0]["results"][0]["codeToUse"];
							$this->smartyVars["showPPCOffer"] = 1;
							$this->smartyVars["ppcCode"] = $data["mbapi"][0]["results"][0]["ppcCodes"][0];
						}
					break;
	
					// code has been activated, now we just display it
					case getAPIStatusID("ppcCodes", "activated"):
						$this->smartyVars["showPPCOffer"] = 1;
						$this->smartyVars["ppcCode"] = $data["mbapi"][0]["results"][0]["ppcCodes"][0];
					break;
		
					// code has been fetched, but the client does not want to display it any longer
					default:
					break;
					}
				}
			}
	
			// ---------------------------------------------------
			// Get OPEN tickets ...
			// ---------------------------------------------------
			$helpdeskEnabled = $CONF["system"]["main"]["helpdesk_enabled"];
			if($helpdeskEnabled) {
				$this->smartyVars["helpdeskEnabled"] = 1;
				$departments = $this->getMBAPIData("GetHelpdeskDepartments", "helpdeskDepartments", "helpdeskDepartment");
				$buckets = $this->getMBAPIData("GetHelpdeskBuckets", "helpdeskBuckets", "helpdeskBucket");
				$query = $this->getMBAPIQuery("GetHelpdeskTickets", "",array(
										"userID" => $clientID, "userType" => 1,
										//	"getHelpdeskDepartmentData" => 1,
										//	"getHelpdeskBucketData" => 1,
										"getUserData" => 1),
										1, 1, 1, 1);
				$data = dispatchMBAPI($query, 1);
		
				if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
					$results =& $data["mbapi"][0]["results"][0];
					$s = array();
					$s["helpdeskDepartments"] =& $departments;
					$s["helpdeskBuckets"] =& $buckets;
					$s["helpdeskTickets"] =& $results["helpdeskTickets"][0]["helpdeskTicket"];
					$c = count($s["helpdeskTickets"]);
					for($i=0; $i<$c; $i++) {
						// Below we add an extra array() to avoid the effects of collapseArrayElements().
						$s["helpdeskTickets"][$i]["hdDepartmentIDs"] =
							array(arrayParentToChild($s["helpdeskTickets"][$i]["hdDepartmentIDs"],
								"hdDepartmentID"));
						$s["helpdeskTickets"][$i]["hdBucketIDs"] =
							array(arrayParentToChild($s["helpdeskTickets"][$i]["hdBucketIDs"],
								"hdBucketID"));
					}
					if(((int) $data["mbapi"][0]["header"][0]["numResults"][0]) === 1) {
						$s["helpdeskTickets"] = array(0 => $results["helpdeskTickets"][0]["helpdeskTicket"][0]);
					}
					if(is_array($s["helpdeskTickets"])) {
						$openID = getAPIStatusID("helpdesk", "open");
						$closedID = getAPIStatusID("helpdesk", "closed");
						$this->smartyVars["helpdeskTicketStatus"] = array("open" => $openID, "closed" => $closedID);
						$ticketStats = array();
						foreach($s["helpdeskTickets"] as $key => $value) {
							switch ($value["hdTicketStatusID"][0]) {
							case 0:
								$this->smartyVars["helpdeskTicketsClosed"][$value["hdTicketID"][0]] = $value["hdTicketKey"][0];
							break;
							case 2:
								$this->smartyVars["helpdeskTicketsOpen"][$value["hdTicketID"][0]] = $value["hdTicketKey"][0];
								$this->smartyVars["helpdeskTicketsOpenSubject"][$value["hdTicketID"][0]] = substr($value["hdTicketSubject"][0], 0, 25);
							break;
							}
						}
					}
					$this->smartyVars["helpdeskTickets"] = collapseArrayElements($s["helpdeskTickets"]);
				}
			} else {
				$this->smartyVars["helpdeskEnabled"] = 0;
			}

			//---------------------------------------------------------------------------------------------
			// Get News
			//---------------------------------------------------------------------------------------------
			$getNews =& new MBQuery("GetNewsGroups", "", array("newsGroupActive" => 1));
			$newsGroups = $getNews->getResult();
			$newsGroups = grabSimpleMBAPIResults( $newsGroups->getStructure() );

			if (is_array( $newsGroups ))
			foreach( $newsGroups as $group ) {
				if (notZero( $group["newsGroupID"] )) {
					$this->smartyVars["news"][] = $group;
					
					$getArticle =& new MBQuery("GetNews", "", array("newsGroupID" => (int)$group["newsGroupID"], "newsActive" => 1));
					$articles = $getArticle->getResult();
					$articles = grabSimpleMBAPIResults( $articles->getStructure() );

					if (notNull( $articles )) {
						$this->smartyVars["news"][ count($this->smartyVars["news"])-1 ]["articles"] = $articles;
					}
					unset( $articles, $getArticle );
				}
			}
			unset($newsGroups, $getNews);
			
			$this->hasExecutedSuccessfully = ((int) $this->addTemplate("dashboard/ShowDashboard", $this->smartyVars));
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // ShowDashboard
?>
