<?php
// $Id: ShowPackages.php,v 1.14 2006/07/02 22:54:27 achrist Exp $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+

require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";

require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-modernbill/include/config/internal/statuses.php");
require_once mb_path("/lib-modernbill/include/config/internal/types.php");
require_once mb_path("/lib-modernbill/include/config/internal/cycles.php");
require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This Action object collects information on all packages that match the given parameters, and 
 * generates the display of that information.
 *
 * @author  getActionFiles.php (Automatically Generated)
 */
class ShowPackages extends MB_Action {
	/**
	 * The constructor for a ShowPackages action object. This merely calls the parent constructor, 
	 * and sets the string representation of this object. This method also sets up $this->params and 
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function ShowPackages() {
		parent::MB_Action();
		$this->stringRep = "ShowPackages";
		$this->params = array();
	}

	/**
	 * The main method of this class that executes the ShowPackages action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;

		dbg($CONF["user"]);

		$rawPackages = collapsePluralsAndSingulars(grabResultsFromMBAPIArray(
			array(
				"mbapi" => array(
					"command" => "GetPackages",
					"params" => array(
						"clientID" => $CONF["user"]["id"],
						"getDomainData" => 1,
						"getPackageAttributeData" => 1
					)
				)
			)
		));
		
		loadConfigs("product", "managementrules");
		
		$this->smartyVars = array();
		$this->smartyVars["managementrules"] = $CONF["product"]["managementrules"];
		$this->smartyVars["numPackages"] = count((array)$rawPackages);
		$this->smartyVars["statusPackages"] = array();

		$this->smartyVars["upgradeDowngradeStatuses"] = array(
			getAPIStatusID("package", "none") => 1,
			getAPIStatusID("package", "pending") => 1,
		);

		foreach ((array)$rawPackages as $k => $package){
		
			//Pull all of the server group IDs out
			$panels = array();
			$attributs = $package["packageAttributes"];
			if(is_array($attributs)) {
				foreach($attributs as $attribute) {
					if($attribute["packageAttributeName"] == "panel_status") {
						$panels[] = $attribute["configGroupID"];
					}
				}
			}

			foreach($panels as $panel) {
				$packageUsage = collapsePluralsAndSingulars(grabResultsFromMBAPIArray(
					array(
						"mbapi" => array(
							"command" => "GetPackageUsageLog",
							"params" => array(
								"packageID" => $package["packageID"],
								"configGroupID" => $panel,
								"usageLogDateBilled" => -1,
								"getTotals" => 1,
				
							)
						)
					)
				));
			}

			$rawPackages[$k]["rules"] = array(
				"addons" => $CONF["product"]["managementrules"]["allowClientAddonManagement"],
				"upgrade" => $CONF["product"]["managementrules"]["allowClientUpgrade"],
				"downgrade" => $CONF["product"]["managementrules"]["allowClientDowngrade"],
			);


			switch ($rawPackages[$k]["packageClientPermissions"]){
				case getAPITypeID("packagemanagement", "upgradedowngradeaddon"):
					$rawPackages[$k]["rules"] = array(
						"addons" => 1,
						"upgrade" => 1,
						"downgrade" => 1,
					);
				break;
				case getAPITypeID("packagemanagement", "upgradedowngrade"):
					$rawPackages[$k]["rules"] = array(
						"addons" => 0,
						"upgrade" => 1,
						"downgrade" => 1,
					);
				break;
				case getAPITypeID("packagemanagement", "upgrade"):
					$rawPackages[$k]["rules"] = array(
						"addons" => 0,
						"upgrade" => 1,
						"downgrade" => 0,
					);
				break;
				case getAPITypeID("packagemanagement", "downgrade"):
					$rawPackages[$k]["rules"] = array(
						"addons" => 0,
						"upgrade" => 0,
						"downgrade" => 1,
					);
				break;
				case getAPITypeID("packagemanagement", "downgradeaddon"):
					$rawPackages[$k]["rules"] = array(
						"addons" => 1,
						"upgrade" => 0,
						"downgrade" => 1,
					);
				break;
				case getAPITypeID("packagemanagement", "upgradeaddon"):
					$rawPackages[$k]["rules"] = array(
						"addons" => 1,
						"upgrade" => 1,
						"downgrade" => 0,
					);
				break;
				case getAPITypeID("packagemanagement", "addon"):
					$rawPackages[$k]["rules"] = array(
						"addons" => 1,
						"upgrade" => 0,
						"downgrade" => 0,
					);
				break;
			}
		
			switch ($rawPackages[$k]["packageStatus"]){
				case getAPIStatusID("package", "none"):
				case getAPIStatusID("package", "pending"):
				case getAPIStatusID("package", "pending_renewal"):
					$rawPackages[$k]["mayCancel"] = 1;
				break;
				default:
					$rawPackages[$k]["mayCancel"] = 0;
				break;
			}
			switch ($rawPackages[$k]["packageCancellationType"]){
				case getAPITypeID("cancellation", "both_suspend"):
				case getAPITypeID("cancellation", "both_delete"):
					$rawPackages[$k]["clientCancel"] = 1;
				break;
				default:
					$rawPackages[$k]["clientCancel"] = 0;
				break;
			}
			$this->smartyVars["statusPackages"][$package["packageStatus"]]["packages"][] =& $rawPackages[$k];
			$this->smartyVars["packageUsage"][$package["packageID"]] =& $packageUsage;
		}

		$this->smartyVars["cancelledStatusID"] = getAPIStatusID("package", "canceled");

		$this->smartyVars["packageStatuses"] = getStatuses("package");
		$this->smartyVars["cycles"] = getCyclesArray();

		dbg($this->smartyVars);
		
		$this->addTemplate("packages/ShowPackages", $this->smartyVars);
			
		$this->hasExecutedSuccessfully = 1;

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // ShowPackages
?>
