<?php
// $Id: AddOrderProduct.php,v 1.25 2006/03/27 18:02:01 achrist Exp $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");
require_once mb_path("/app-modernbill-order/include/order/order.php");

/**
 * This Action object collects information on all orderForms that match the given parameters, and 
 * generates the display of that information.
 *
 * @author  Mark Garrett
 */
class AddOrderProduct extends MB_Action {
	/**
	 * The constructor for a AddOrderProduct action object. This merely calls the parent constructor, 
	 * and sets the string representation of this object. This method also sets up $this->params and 
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function AddOrderProduct() {
		parent::MB_Action();
		$this->stringRep = "AddOrderProduct";
		$this->params = array(
			"sid",
			"cycleID",
			"productID",
			"domainName",
			"couponCode",
			"domainLanguage",
			"set_locale",
			"ordOnly"
			);
		$this->sortColumns = array();
	}

	/**
	 * The main method of this class that executes the AddOrderProduct action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();
		
		$ofdata = NULL;
		
		$CONF["theme"]["type"] = "public";

		if($_COOKIE["orderobj"]){
			$orderobj = unserialize(gzuncompress(base64_decode($_COOKIE["orderobj"])));
		}
		
		$orderobj->orderFormSessionID = $this->paramArgs["sid"];

		if($this->paramArgs["productID"] || $this->paramArgs["domainName"]) {

			$smartyVars = array();
			
			// Do the product add
			$orderProducts = $orderobj->setOrderProducts((int)$this->paramArgs["productID"], (int)$this->paramArgs["cycleID"],
				$this->paramArgs["domainName"], $this->paramArgs["couponCode"], $this->paramArgs["domainLanguage"]);

			$orderobj->orderFormData = NULL;
			setcookie("orderobj", base64_encode(gzcompress(serialize($orderobj))));
			// Get the current prod data from the order to display
		}
		if (!(int)$this->paramArgs["ordOnly"]){
			
			// Get products to display for ordering - does not include domain products
			$orderProdData = $orderobj->getProductInfo();
			
			$orderProdData =(is_array($orderProdData)) ? $orderProdData : array();
			
			// Go through the products and their prices and create smarty-friendly select menues for prices
			if (count($orderProdData)){
				require_once mb_path("/lib-modernbill/include/config/internal/cycles.php");
				foreach($orderProdData as $prodID => $prodData){
					if (is_array($prodData["prices"])){
						foreach($prodData["prices"] as $key => $prodPrice){
							$cycleDN = getCycleName($key);
							$cycleDisplayName = (defined($cycleDN)) ? constant($cycleDN) : $cycleDN;
							$orderProdData[$prodID]["priceSelectMenu"][$key] = $cycleDisplayName." - ".$CONF["user"]["locale"]->getLocaleCurrency($prodPrice["price"], $orderobj->orderFormCurrency);
							
							if ((float)$prodPrice["setup"] > 0.00){
								$orderProdData[$prodID]["priceSelectMenu"][$key] .= " + ".TRANS_SETUP.": ".$CONF["user"]["locale"]->getLocaleCurrency($prodPrice["setup"], $orderobj->orderFormCurrency);
							}
						}
					}
				}
			}

			// Are there coupons that can be associated with this order form?
			if(is_array($orderobj->orderFormData["coupons"][0]["coupon"])){
				$smartyVars["showCouponField"] = TRUE;
			}
			
			//Clean up product key values
			if (count($orderProdData)){
				$smartyVars["products"] = protectArray(array_values($orderProdData));
			}
			
			//List of potential domain names and get them to smarty
			$domainNames = $orderobj->getOrderDomains();

			foreach ($domainNames as $key=>$value){
				$smartyVars["domainNames"][$value] = $value;
			}
			
			//List of order parameters
			$smartyVars["orderParams"] = $orderobj->getOrderParams();

		}			
		// Get the current order object products data to display - these are are already ordered objects
		$orderItemData = array();
		$orderItemData = $orderobj->getOrderItems();
		
		$smartyVars["sid"] = $orderobj->orderFormSessionID;
		$smartyVars["set_locale"] = $orderobj->orderFormLocale;
		$orderobj->orderFormData = NULL;
		setcookie("orderobj", base64_encode(gzcompress(serialize($orderobj))));


		// Set the display price - right now does coupon types 0 (percent) and 1 (unit). Additional ifelse statements
		// will need to be added if more type are created
		
		$grandTotal = NULL;

		foreach($orderItemData as $key=>$item){
			if(is_array($item["coupon"])){
				if($item["coupon"]["type"] == 1){
					$orderItemData[$key]["finalPrice"] = $orderItemData[$key]["prices"]["price"] - $item["coupon"]["amount"];
					$orderItemData[$key]["couponDisplay"][] = $item["coupon"]["code"]."-".$CONF["user"]["locale"]->getLocaleCurrency($item["coupon"]["amount"], $orderobj->orderFormCurrency);
				}elseif($item["coupon"]["type"] == 0){
					$orderItemData[$key]["finalPrice"] = $orderItemData[$key]["prices"]["price"] - (($item["coupon"]["amount"]/100) * $orderItemData[$key]["prices"]["price"]);
				$orderItemData[$key]["couponDisplay"][] = $item["coupon"]["code"]."-".$CONF["user"]["locale"]->getLocalePercent($item["coupon"]["amount"], $orderobj->orderFormCurrency);
				}
			}else{
				$orderItemData[$key]["finalPrice"] = $orderItemData[$key]["prices"]["price"];
			}

			$orderItemData[$key]["finalPrice"] = $orderItemData[$key]["finalPrice"] + $orderItemData[$key]["prices"]["setup"];
		
			// Just a quick check to make sure that if there are too many coupons applied - it doesn't go less than 0
			if ((float)$orderItemData[$key]["finalPrice"] < 0){
				$orderItemData[$key]["finalPrice"] = 0.00;
			}
			$grandTotal += $orderItemData[$key]["finalPrice"];
		
			// We go through the products that we have in the order and look for domain packages that aren't transfer.
			// We then pass those through to see if we need to get any additional data for the tld
			$domainType = getTypeID("products", TRANS_DOMAIN_PRODUCT);
			foreach ($orderobj->orderProducts as $ordprod){
				if($item["productType"] == $domainType && $ordprod["cycleID"] > 0 && $item["productID"] == $ordprod["productID"]){
					$domdata = getTLDParamsData($ordprod["domainName"]);
					if ($domdata){
						$smartyVars["extraData"][$ordprod["domainName"]] = $domdata;
						$domdata = NULL;
					}
				}
			}		
		}

		$smartyVars["grandTotal"] =$grandTotal;
		$smartyVars["orderItems"] =protectArray($orderItemData);
		$smartyVars["currency"] =$orderobj->orderFormCurrency;
		$smartyVars["ofDir"] =$orderobj->orderFormDir;
   		$smartyVars["ofCssDir"] =$CONF["relativeWebRoot"]."/app-modernbill-order/include/themes/".$CONF["theme"]["name"];

		
		// Get country list for customer info input. Pass in the locale country to get it's ID
		$this->mergeCountryData($CONF["user"]["locale"]->country);
		$smartyVars["countryList"] = $this->smartyVars["countryList"];
		$smartyVars["selectedCountry"] = $this->smartyVars["selectedCountry"];

		// If we have domains on the order form, we need to get the prospective whois info 
		if (count($orderobj->orderDomains) && count($orderobj->domainProducts)){
			$smartyVars["getTLDWhoisData"] = TRUE;
		}

		$smartyVars = collapseArrayElements($smartyVars);
		
		if (!(int)$this->paramArgs["ordOnly"]){
			$this->addTemplate($orderobj->orderFormDir."/widgets/displayOrderForm", $smartyVars);
		}else{
			$this->addTemplate($orderobj->orderFormDir."/widgets/yourOrder", $smartyVars);
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
} // AddOrderProduct
?>