<?php
// $Id: order.php,v 1.38 2006/05/23 19:44:30 kbailey Exp $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+

if(!$CONF || $_REQUEST["CONF"]) {
	require_once "../common.php";
}

require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This class is an instantion of the Command class, that executes a command pertaining to
 * processing order information, in the default case from the database. This class should
 * only be instantiated by dispatchMBAPI() found in include/mbapi/mbapi.php.
 *
 * Something similar to the following XML should be passed to dispatchMBAPI() when you want
 * to call the ProcessOrder command:
 *
 * @author Mark Garrett
 */
class Order {

	/**
	 * Variables to collect to use in generating an order. Each of these will be an array
     * of the indicated items.
	 */

	/**
	 * This is an array of the checked domains that have been requested. The product info
     * for these domains will be in orderProducts. This will just be the SLD plus TLD. It
     * will be used when selecting products to associate a domain with that product.
	 */
	var $orderDomains;

	/**
	 * This is an array of the products that have been added to the order. This is an
     * associative array that will include the productID, cycleID, and associated
     * orderDomains key (if any)
	 */
	var $orderProducts;

	/**
	 * This is an array of the coupon codes that have been added to the order. This array
     * has been checked to see if it is applicable to this order form and possible products.
     * They will be applied to relevant items.
	 */
	var $orderCoupons;

	/**
	 * This is an array of the CRM data collected with the order form. 
	 */
	var $orderCRM;

	/**
	 * This is an array of the domain management data collected with the order form. This
     * would include additional CIRA style info, etc.
	 */
	var $orderDomainManagement;

	/**
	 * The order form id. We will need this in various places.
	 */
	var $orderFormID;

	/**
	 * The order form directory where the tempaltes are stored. This does not account for the theme - that will be the 
	 * Order Admin's theme
	 */
	var $orderFormDir;

	/**
	 * The order form data.
	 */
	var $orderFormData;

	/**
	 * The order form locale.
	 */
	var $orderFormLocale;

	/**
	 * The order form languageID.
	 */
	var $orderFormLanguageID;

	/**
	 * The order form currency.
	 */
	var $orderFormCurrency;

	/**
	 * The order form session ID.
	 */
	var $orderFormSessionID;

	/**
	 * The order form products that are Domain Registration products.
	 */
	var $domainProducts;

	/**
	 * The step of the order process we are in - insert, update, process, or install.
	 */
	var $processStep;

	/**
	 * The affiliate code to be associated with packages in this order.
	 */
	var $affiliateCode;

	/**
	 * Whether or not we are to check the "free" email banning table to validate email address.
	 */
	var $emailBan;

	/**
	 * Continue Shopping URL.
	 */
	var $continueShoppingURL;

	/**
	 * The text for email banning.
	 */
	var $emailBanText;

	/**
	 * The once we have inserted the client, we need to store the ID (blowfish encrypted) so that we can update that 
	 * client if needs be.
	 */
	var $clientID;

	function Order($ordparams = NULL) {
		
		global $CONF;
		
		$this->orderDomains = $this->orderProducts = $this->orderCoupons = array();
		$this->orderFormID = (int)$ordparams["ofid"];
		$this->orderFormSessionID = $ordparams["sid"];
		$this->orderFormLocale = $ordparams["set_locale"];
		$this->affiliateCode = $ordparams["afcode"];

		if (!is_array($ordparams["ofdata"])){
		// Get the info for this order form
			$getOptions = array(
				"remoteSessionID" => $this->orderFormSessionID,
				"getOrderFormParams" => 1,
				"getOrderFormItems" => 3,
				"getCouponData" => 1,
				"getOrderFormProductAddons" => 1,
				"orderFormID" => $this->orderFormID
			);

			$query = getMBAPIQuery("GetOrderForms", "", $getOptions);
			$data = dispatchMBAPI($query, 1);
			$this->orderFormSessionID =& $data["mbapi"][0]["header"][0]["remoteSessionID"][0];

			$this->orderFormData =& $data["mbapi"][0]["results"][0]["orderForms"][0]["orderForm"][0];
		}else{
			$this->orderFormData = $ordparams["ofdata"];
		}

		// See if we want to check email ban table
		
		foreach($this->orderFormData["orderFormParams"][0]["orderFormParam"] as $param){
			if($param["orderFormParamName"][0] == "email_banning"){
				$this->emailBan = $param["orderFormParamValue"][0];
			}elseif($param["orderFormParamName"][0] == "email_banning_text"){
				$this->emailBanText = $param["orderFormParamValue"][0];
			}elseif($param["orderFormParamName"][0] == "ip_banning_text"){
				$this->ipBanText = $param["orderFormParamValue"][0];
			}elseif($param["orderFormParamName"][0] == "continue_shopping_url"){
				$this->continueShoppingURL = $param["orderFormParamValue"][0];
			}elseif($param["orderFormParamName"][0] == "dom_options"){
				$this->domainOptions = unserialize($param["orderFormParamValue"][0]);
			}
		}
		
		// Get the order form template directory
		if (file_exists("../include/themes/".$CONF["theme"]["name"]."/templates/order".$this->orderFormID)) {
			$this->orderFormDir = "order".$this->orderFormID;
		}else{
			$this->orderFormDir = "order";
		}

		$this->orderFormCurrency = $this->orderFormData["paymentGroupCurrencyID"][0];

		if ($ordparams["domainName"] || $ordparams["productID"]){
			$this->setOrderProducts($ordparams["productID"], $ordparams["cycleID"], $ordparams["domainName"], $ordparams["couponCode"], $ordparams["domainLanguage"]);
		}

        // Display an error if no order form data can be found. The most likely cause is
        // that an order form has not been setup yet. - MF
        if (!is_array($this->orderFormData["orderFormItems"])) {
            Error::fatal(TRANS_ORDERFORMERROR, "Failed to find order form data.");
        }

	}

	/**
	 * The function that adds products to the order object. This includes adding coupon codes
     * that have been entered.  The productIDs and couponCodes are checked to see if they are
     * applicable to this order form. A final check on productIDs will also be performed when
     * the "soft" add occurs as a prevent against cookie hacking.
	 */
	function setOrderProducts($productIDs, $cycleIDs, $domainNames, $couponCodes, $domainLanguages){
		$productIDs = ($productIDs) ? (array)$productIDs : NULL;
		$cycleIDs = (array)$cycleIDs;
		$domainNames = (array)$domainNames;
		$couponCodes = (array)$couponCodes;
		$domainLanguages = (array)$domainLanguages;

		// Check to make sure is is in form - also get additional data as we will be calling
        // displayOrderItems right after this
		if (!is_array($this->orderFormData) && count($productIDs)){

		// Get the info for this order form
			$getOptions = array(
				"remoteSessionID" => $this->orderFormSessionID,
				"getOrderFormItems" => 3,
				"getOrderFormProductAddons" => 1,
				"getCouponData" => 1,
				"getOrderFormParams" => 1,
				"orderFormID" => $this->orderFormID
			);

			$query = getMBAPIQuery("GetOrderForms", "", $getOptions);
			$data = dispatchMBAPI($query, 1);

			$this->orderFormSessionID =& $data["mbapi"][0]["header"][0]["remoteSessionID"][0];

			$this->orderFormData =& $data["mbapi"][0]["results"][0]["orderForms"][0]["orderForm"][0];
		}

		if(is_array($this->orderFormData["coupons"][0]["coupon"])){
			foreach($this->orderFormData["coupons"][0]["coupon"] as $coup){
				$possibleCoupons[] = $coup["couponCode"][0];
			}

			foreach($couponCodes as $key=>$code){
				if (in_array($code, $possibleCoupons) && !in_array($code, $this->orderCoupons)){
					$this->orderCoupons[] = $code;
				}
			}
		}

		if(is_array($this->orderFormData["orderFormItems"][0]["orderFormItem"])){
			foreach($this->orderFormData["orderFormItems"][0]["orderFormItem"] as $ofprod){
				$possibleProducts[] = $ofprod["productID"][0];
			}
		}

		$next = NULL;
		if (count($productIDs)){
		 	dbg("adding products into order: ");
			foreach($productIDs as $key=>$prodID){
				if (in_array($prodID, $possibleProducts)){
				 	dbg("added product: ".$prodID);
					$next = count($this->orderProducts)+1;
					$this->orderProducts[$next]["productID"] = $prodID;
					$this->orderProducts[$next]["cycleID"] = $cycleIDs[$key];
					$this->orderProducts[$next]["domainName"] = $domainNames[$key];
					$this->orderProducts[$next]["domainLanguage"] = $domainLanguages[$key];
					if(!in_array($domainNames[$key], $this->orderDomains) && $domainNames[$key] != ""){
					 	dbg("added prod domain: ".$domainNames[$key]);
						$this->orderDomains[] = $domainNames[$key];
					}
					$next = NULL;
				}
			}
		}elseif(count($domainNames)){
		 	dbg("adding domain names into order:");
			foreach($domainNames as $key=>$name){
				if(!in_array($name, $this->orderDomains)){
				 	dbg("added domain: ".$name);
					$this->orderDomains[] = $name;
				}
			}
		}
	}


	/**
	 * The function that remove products to the order object.
	 */
	function removeOrderProducts($productID, $cycleID, $domainName){
		
		// If this is a domain product that is being removed - remove from orderDomains and the name from all other products
		if (in_array($productID, $this->domainProducts)){
			$domainKey = array_keys($this->orderDomains, $domainName);
			unset($this->orderDomains[$domainKey[0]]);
			$removeDomainName = TRUE;
		}
		
		//Find the specific package that needs to be removed - must match all three criteria. Set the domain name to null
		//for any other packge if we are removing the dom reg product
		foreach ($this->orderProducts as $key=>$prod){
			if ($prod["productID"] === $productID && $prod["cycleID"] === $cycleID && $prod["domainName"] == $domainName) {
				unset($this->orderProducts[$key]);
			}elseif($prod["domainName"] === $domainName && $removeDomainName){
				$this->orderProducts[$key]["domainName"] = NULL;
			}
		}
		
		$this->orderProducts = array_values($this->orderProducts);

	}

	/**
	 * This function obtains all the TLD info for Domain Registration products that are associated
     * with this order form. It optionally returns pricing data.
	 *
	 */
	function getDomainInfo($pricedataTLD = FALSE) {

		// Must get form data if we don't have it - Used for those who come back to the OF with a cookie already
		if (!is_array($this->orderFormData)){
			// Get the info for this order form
			$getOptions = array(
				"getOrderFormParams" => 1,
				"getOrderFormItems" => 3,
				"orderFormID" => $this->orderFormID
			);

			$query = getMBAPIQuery("GetOrderForms", "", $getOptions);
			$data= dispatchMBAPI($query, 1);
			$this->orderFormSessionID =& $data["mbapi"][0]["header"][0]["remoteSessionID"][0];
			$this->orderFormData =& $data["mbapi"][0]["results"][0]["orderForms"][0]["orderForm"][0];
		}

		// If I don't already have an array of domain products, figure out what they are from the of items
		if (!is_array($this->domainProducts)){
			$domainType = getTypeID("products", TRANS_DOMAIN_PRODUCT);
			foreach($this->orderFormData["orderFormItems"][0]["orderFormItem"] as $ofItem){
				if ($ofItem["productType"][0] == $domainType) {
					$this->domainProducts[] = $ofItem["productID"][0];
				}
			}
		}
		
		if (!count($this->domainProducts)){
			$this->domainProducts = array();
		}

		if ($pricedataTLD){
			$getData = "<getTLDData>1</getTLDData>\n
						<getCycleData>1</getCycleData>\n
						<getProductVariantData>1</getProductVariantData>\n
						<getProductVariantPriceData>1</getProductVariantPriceData>";
		}else{
			$getData = "<getTLDData>1</getTLDData>";
		}

		//Only do a look up if there are domain products associated with the order form
		if (count($this->domainProducts)){
			// Go get all the TLD price and cycle info
			$cmd = "<mbapi>
			<remoteSessionID>".$this->orderFormSessionID."</remoteSessionID>
			<command>GetProducts</command>
			<params>
					<productID>".implode($this->domainProducts, "</productID>\n<productID>")."</productID>
					<orderFormID>".$this->orderFormID."</orderFormID>
					".$getData."
			</params>
		</mbapi>";
	
			$domainInfo = $temp = array();
	
			//Get TLD list

			$data= dispatchMBAPI($cmd, 1);
			dbg("GetProducts ---- Get Domain Info - ".__LINE__." - ".$cmd);
			dbg("GetProducts ---- Get Domain Info Result- ".__LINE__);
			dbg($data);
			$this->orderFormSessionID =& $data["mbapi"][0]["header"][0]["remoteSessionID"][0];
			$prodData = (array)$data["mbapi"][0]["results"][0]["products"][0]["product"];

			foreach($prodData as $prod){

				// Get array of TLD info to display pop-up of available TLDs
				if(is_array($prod["TLDs"][0]["TLD"])){
					foreach($prod["TLDs"][0]["TLD"] as $tld){
						dbg("TLD");
						dbg($tld);
						if ($tld["tldActive"][0] && $tld["productVariantActive"][0]){
							$domainInfo["tld"][$tld["tldExtension"][0]] = $tld["tldExtension"][0];
							$domainInfo["idn"][$tld["tldExtension"][0]] = $tld["tldIDNSupport"][0];
							$tldInfo[$tld["tldID"][0]] = $tld;
						}
					}
				}

				// Set an array of prod var IDs with the value as the associated TLD ID
				if(is_array($prod["productVariants"][0]["productVariant"])){
					foreach($prod["productVariants"][0]["productVariant"] as $variant){
						dbg("variant");
						dbg($variant);
						$prodVarTLD[$variant["productVariantID"][0]] = $variant["tldID"][0];
					}
				}
				
				// Prep the Cycles
				if(is_array($prod["cycles"][0]["cycle"])){
					foreach($prod["cycles"][0]["cycle"] as $cycle){
						dbg("Cycle");
						dbg($cycle);
						$cycleData[$cycle["cycleID"][0]] = $cycle;
					}
				}


				//Loop through the variant prices
				if(is_array($prod["productVariantPrices"][0]["productVariantPrice"])){
					foreach($prod["productVariantPrices"][0]["productVariantPrice"] as $variantPrice){
						dbg("VariantPrice");
						dbg($variantPrice);
						$domainInfo["pricing"][$tldInfo[$prodVarTLD[$variantPrice["productVariantID"][0]]]["tldExtension"][0]][$variantPrice["cycleID"][0]]["tldIsTransferable"] = $tldInfo[$prodVarTLD[$variantPrice["productVariantID"][0]]]["tldIsTransferable"][0];
						$domainInfo["pricing"][$tldInfo[$prodVarTLD[$variantPrice["productVariantID"][0]]]["tldExtension"][0]][$variantPrice["cycleID"][0]]["tldTransferMinimumCycleID"] = $tldInfo[$prodVarTLD[$variantPrice["productVariantID"][0]]]["tldTransferMinimumCycleID"][0];
						$domainInfo["pricing"][$tldInfo[$prodVarTLD[$variantPrice["productVariantID"][0]]]["tldExtension"][0]][$variantPrice["cycleID"][0]]["productID"] = $prod["productID"][0];
						$domainInfo["pricing"][$tldInfo[$prodVarTLD[$variantPrice["productVariantID"][0]]]["tldExtension"][0]][$variantPrice["cycleID"][0]]["productVariantID"] = $variantPrice["productVariantID"][0];

						if ($variantPrice["productVariantPriceType"][0] == 1){
							$domainInfo["pricing"][$tldInfo[$prodVarTLD[$variantPrice["productVariantID"][0]]]["tldExtension"][0]][$variantPrice["cycleID"][0]]["productSetupPrice"] = $variantPrice["productVariantPriceAmount"][0];
						}elseif ($variantPrice["productVariantPriceType"][0] == 2){
							$domainInfo["pricing"][$tldInfo[$prodVarTLD[$variantPrice["productVariantID"][0]]]["tldExtension"][0]][$variantPrice["cycleID"][0]]["productTransferPrice"] = $variantPrice["productVariantPriceAmount"][0];
						}elseif ($variantPrice["productVariantPriceType"][0] == 0){
							$domainInfo["pricing"][$tldInfo[$prodVarTLD[$variantPrice["productVariantID"][0]]]["tldExtension"][0]][$variantPrice["cycleID"][0]]["productPrice"] = $variantPrice["productVariantPriceAmount"][0];
						}
						$domainInfo["pricing"][$tldInfo[$prodVarTLD[$variantPrice["productVariantID"][0]]]["tldExtension"][0]][$variantPrice["cycleID"][0]]["cycleID"] = $variantPrice["cycleID"][0];
						$domainInfo["pricing"][$tldInfo[$prodVarTLD[$variantPrice["productVariantID"][0]]]["tldExtension"][0]][$variantPrice["cycleID"][0]]["cycleDisplayName"] = $cycleData[$variantPrice["cycleID"][0]]["cycleDisplayName"][0];
					}
				}	
			}
		}

		foreach($this->orderFormData["orderFormParams"][0]["orderFormParam"] as $param){
			if($param["orderFormParamName"][0] == "dom_options" && $param["orderFormParamValue"][0]){
				$temp = unserialize($param["orderFormParamValue"][0]);
				array_unshift($temp, "padding");
				$domainInfo["domOptions"] = array_flip($temp);
				$domainInfo["optionsNumber"] = count($domainInfo["domOptions"]) - 1; // Number of table rows to display - starts at 1, not 0
			}
		}

		return $domainInfo;
	}

	function getOrderDomains() {
		return $this->orderDomains;
	}


	/**
	 *	This function obtains all the product info for products that are associated with this order form. This is used to
	 *	display products, their cycles and prices and associated addons. Does not return Domain Registration products
	 *
	 */
	function getProductInfo() {

		if (!is_array($this->orderFormData)){
		// Get the info for this order form
			$getOptions = array(
				"remoteSessionID" => $this->orderFormSessionID,
				"getOrderFormItems" => 3,
				"getOrderFormProductAddons" => 1,
				"getCouponData" => 1,
				"getOrderFormParams" => 1,
				"orderFormID" => $this->orderFormID
			);

			$query = getMBAPIQuery("GetOrderForms", "", $getOptions);
			$data = dispatchMBAPI($query, 1);
			$this->orderFormSessionID =& $data["mbapi"][0]["header"][0]["remoteSessionID"][0];

			$this->orderFormData =& $data["mbapi"][0]["results"][0]["orderForms"][0]["orderForm"][0];
		}

		//List all products so we can get prices - remove DomReg products
		$domainType = getTypeID("products", TRANS_DOMAIN_PRODUCT);
		
		$productList = array();
		if (is_array($this->orderFormData["orderFormItems"][0]["orderFormItem"])){
			foreach($this->orderFormData["orderFormItems"][0]["orderFormItem"] as $ofItem){
				if ($ofItem["productType"][0] != $domainType) {
					$productList[] = $ofItem["productID"][0];
					$products[$ofItem["productID"][0]] = $ofItem;
				}
			}
		}

		//List all possible productAddons for this order form
		if (is_array($this->orderFormData["productAddons"][0]["productAddon"])){
			foreach($this->orderFormData["productAddons"][0]["productAddon"] as $ofAddon){
				if (in_array($ofAddon["productID"][0], $productList)) {
					$productAddonList[] = $ofAddon["productAddonID"][0];
				}
			}
		}
		if (count($productList)){
			$pricequery = "<mbapi>
			<remoteSessionID>".$this->orderFormSessionID."</remoteSessionID>
			<command>GetProducts</command>
			<params>
				<productID>".implode($productList, "</productID>\n<productID>")."</productID>
				<getCycleData>1</getCycleData>
				<getProductVariantData>1</getProductVariantData>
				<getProductVariantPriceData>1</getProductVariantPriceData>
				<orderFormID>".$this->orderFormID."</orderFormID>
			</params>
		</mbapi>";
	
	/*          <productAddonIDs>
					 <productAddonID>".implode($productAddonList, "</productAddonID>\n<productAddonID>")."</productAddonID>
				</productAddonIDs>
	*/
	
			$pdata = dispatchMBAPI($pricequery, 1);
			$this->orderFormSessionID =& $pdata["mbapi"][0]["header"][0]["remoteSessionID"][0];
	
			$priceData =& $pdata["mbapi"][0]["results"][0]["products"][0]["product"];
			if(is_array($priceData)){
				foreach($priceData as $prod){
					// Prep the Cycles
					if(is_array($prod["cycles"][0]["cycle"])){
						foreach($prod["cycles"][0]["cycle"] as $cycle){
							$cycleData[$cycle["cycleID"][0]] = $cycle;
						}
					}
	
					//Loop through the variant prices
					if(is_array($prod["productVariantPrices"][0]["productVariantPrice"])){
						foreach($prod["productVariantPrices"][0]["productVariantPrice"] as $variantPrice){
							if ($variantPrice["productVariantPriceType"][0] == 1){
								$products[$prod["productID"][0]]["prices"][$variantPrice["cycleID"][0]]["setup"] = $variantPrice["productVariantPriceAmount"][0];
							}elseif ($variantPrice["productVariantPriceType"][0] == 0){
								$products[$prod["productID"][0]]["prices"][$variantPrice["cycleID"][0]]["price"] = $variantPrice["productVariantPriceAmount"][0];
							}
						}
					}	
				}
			}
		}
		
		return $products;
	}

	/**
	 *	This function obtains all the info for products that have been ordered. This is used to
	 *	display the order as it currently exists.
	 *
	 */
	function getOrderItems() {

		if (!is_array($this->orderFormData) && count($this->orderProducts)){

		// Get the info for this order form
			$getOptions = array(
				"remoteSessionID" => $this->orderFormSessionID,
				"getOrderFormItems" => 2,
				"getOrderFormProductAddons" => 1,
				"getCouponData" => 1,
				"orderFormID" => $this->orderFormID
			);

			$query = getMBAPIQuery("GetOrderForms", "", $getOptions);
			$data = dispatchMBAPI($query, 1);
			$this->orderFormSessionID =& $data["mbapi"][0]["header"][0]["remoteSessionID"][0];

			$this->orderFormData =& $data["mbapi"][0]["results"][0]["orderForms"][0]["orderForm"][0];
		}

		$coupondata = array();
		// Get array of coupons with the code as the key
		$coupondata =& $this->orderFormData["coupons"][0]["coupon"];

		$prodIDs = array();
		foreach ($this->orderProducts as $prodArray){
			if (!in_array($prodArray["productID"], $prodIDs)){
				$prodIDs[] = $prodArray["productID"];
			}
		}

		dbg("products we are going to get:");
		dbg($prodIDs);

		//Get prices for the products - we get all the cycles, but no mechanism right now to get only 1 
		$pricequery = "<mbapi>
        <remoteSessionID>".$this->orderFormSessionID."</remoteSessionID>
        <command>GetProducts</command>
        <params>
            <productID>".implode($prodIDs, "</productID>\n<productID>")."</productID>
            <getTLDData>1</getTLDData>
			<getCycleData>1</getCycleData>
			<getProductVariantData>1</getProductVariantData>
			<getProductVariantPriceData>1</getProductVariantPriceData>
			<orderFormID>".$this->orderFormID."</orderFormID>
        </params>
	</mbapi>";

/*          <productAddonIDs>
                 <productAddonID>".implode($productAddonList, "</productAddonID>\n<productAddonID>")."</productAddonID>
			</productAddonIDs>
*/
		$pdata = dispatchMBAPI($pricequery, 1);
		$this->orderFormSessionID =& $pdata["mbapi"][0]["header"][0]["remoteSessionID"][0];

		$prodData =& $pdata["mbapi"][0]["results"][0]["products"][0]["product"];

		dbg("products we got from those id's");
		dbg($prodData);

		$productList = array();
		if (is_array($prodData)){
			foreach($prodData as $prod){
				$productList[] = $prod["productID"][0];
				
				// Get array of TLD info to display pop-up of available TLDs
				if(is_array($prod["TLDs"][0]["TLD"])){
					foreach($prod["TLDs"][0]["TLD"] as $tld){
						if ($tld["tldActive"][0]){
							$productsData[$prod["productID"][0]]["tldInfo"][$tld["tldID"][0]] = $tld;
						}
					}
				}
				
				// Set an array of prod var IDs with the value as the associated TLD ID
				if(is_array($prod["productVariants"][0]["productVariant"])){
					foreach($prod["productVariants"][0]["productVariant"] as $variant){
						$productsData[$prod["productID"][0]]["prodVarTLD"][$variant["productVariantID"][0]] = $productsData[$prod["productID"][0]]["tldInfo"][$variant["tldID"][0]];
					}
				}
				
				// Prep the Cycles
				if(is_array($prod["cycles"][0]["cycle"])){
					foreach($prod["cycles"][0]["cycle"] as $cycle){
						$productsData[$prod["productID"][0]]["cycleData"][$cycle["cycleID"][0]] = $cycle;
					}
				}
				
				$productsData[$prod["productID"][0]] += $prod;
				unset($productsData[$prod["productID"][0]]["TLDs"]);
				unset($productsData[$prod["productID"][0]]["cycles"]);
				unset($productsData[$prod["productID"][0]]["productVariants"]);
				unset($productsData[$prod["productID"][0]]["tldInfo"]);
				$productsData[$prod["productID"][0]]["productVariantPrices"] = $productsData[$prod["productID"][0]]["productVariantPrices"][0]["productVariantPrice"];
			}
		}

		//List all possible productAddons for this order form
		if (is_array($this->orderFormData["productAddons"][0]["productAddon"])){
			foreach($this->orderFormData["productAddons"][0]["productAddon"] as $ofAddon){
				if (in_array($ofAddon["productID"][0], $productList)) {
					$productAddons[$ofAddon["productAddonID"][0]] = $ofAddon;
				}
			}
		}

		//Set Display Array
		$orderDisplay = array();
		foreach ($this->orderProducts as $prodKey=>$orderArray){
			$next = count($orderDisplay);
			$orderDisplay[$next]["productName"] = $productsData[$orderArray["productID"]]["productName"][0];
			$orderDisplay[$next]["productType"] = $productsData[$orderArray["productID"]]["productType"][0];
			$orderDisplay[$next]["domainName"] = $orderArray["domainName"];
			$orderDisplay[$next]["productID"] = $orderArray["productID"];
			$orderDisplay[$next]["cycleID"] = $orderArray["cycleID"];
			
			if(is_array($productsData)){
				foreach($productsData as $product){

					// Do non-domain products first
					if (($product["productID"][0] == $orderArray["productID"]) 
							&& (!in_array($orderArray["productID"], $this->domainProducts)) 
							&& is_array($product["productVariantPrices"])){
						
						foreach ($product["productVariantPrices"] as $price){
							if($price["productVariantPriceType"][0] == 0 && $price["cycleID"][0] == $orderArray["cycleID"]){
								$orderDisplay[$next]["prices"]["price"] = $price["productVariantPriceAmount"][0];
								$orderDisplay[$next]["cycleDisplayName"] = $product["cycleData"][$orderArray["cycleID"]]["cycleDisplayName"][0];
							}elseif($price["productVariantPriceType"][0] == 1 && $price["cycleID"][0] == $orderArray["cycleID"] && (float) $price["productVariantPriceAmount"][0] > 0.00){
								$orderDisplay[$next]["prices"]["setup"] = $price["productVariantPriceAmount"][0];
							}
						}

					// Do Domain products
					}elseif ($product["productID"][0] == $orderArray["productID"] && is_array($product["productVariantPrices"])){
						$domTLD = $exploded = NULL;
						$exploded = explode(".", $orderArray["domainName"]);
						array_shift($exploded);
						$domTLD = implode(".", $exploded);
						
						foreach ($product["prodVarTLD"] as $prodVarID => $tld){
							if($tld["tldExtension"][0] == $domTLD && $tld["tldActive"][0]){
								foreach ($product["productVariantPrices"] as $price){
									if($price["productVariantPriceType"][0] == 0 && $price["cycleID"][0] == $orderArray["cycleID"] && $price["productVariantID"][0] == $prodVarID){
										$orderDisplay[$next]["prices"]["price"] = $price["productVariantPriceAmount"][0];
										$orderDisplay[$next]["cycleDisplayName"] = $product["cycleData"][$orderArray["cycleID"]]["cycleDisplayName"][0];
									}elseif($price["productVariantPriceType"][0] == 1 && $price["cycleID"][0] == $orderArray["cycleID"] && $price["productVariantID"][0] == $prodVarID && (float) $price["productVariantPriceAmount"][0] > 0.00){
										$orderDisplay[$next]["prices"]["setup"] = $price["productVariantPriceAmount"][0];
									}elseif($price["productVariantPriceType"][0] == 2 && $orderArray["cycleID"] == '-2' && $price["productVariantID"][0] == $prodVarID){
										$orderDisplay[$next]["prices"]["price"] = $price["productVariantPriceAmount"][0];
										$orderDisplay[$next]["cycleDisplayName"] = TRANS_TRANSFER;										
									}elseif($price["productVariantPriceType"][0] == 1 && $orderArray["cycleID"] == '-2' && $price["productVariantID"][0] == $prodVarID && (float) $price["productVariantPriceAmount"][0] > 0.00){
										$orderDisplay[$next]["prices"]["setup"] = $price["productVariantPriceAmount"][0];
									}
								}
							}
						}
					}
				}
			}

			$orderDisplay[$next]["cycleDisplayName"] = (defined($orderDisplay[$next]["cycleDisplayName"])) ? constant($orderDisplay[$next]["cycleDisplayName"]) : $orderDisplay[$next]["cycleDisplayName"];

			if(is_array($coupondata)){
				foreach($coupondata as $ofCoupon){
					if (in_array($ofCoupon["couponCode"][0], $this->orderCoupons) && $orderArray["productID"] == $ofCoupon["productID"][0]){
						$orderDisplay[$next]["coupon"] = array("code"=>$ofCoupon["couponCode"][0], "amount"=>$ofCoupon["couponDiscount"][0], "type"=>$ofCoupon["couponType"][0]);
						break;
					}
				}
			}

			if ($orderDisplay[$next]["prices"]["price"] == NULL){
				unset($this->orderProducts[$prodKey]);
				unset($orderDisplay[$next]);
			}

		}
		return $orderDisplay;
	}

	/**
	 *	This function returns specific information about the parameters for this order form such as email banning, password
	 *  and username lengths, etc.
	 *
	 */
	function getOrderParams() {

		if (!is_array($this->orderFormData["orderFormParams"])){

		// Get the info for this order form
			$getOptions = array(
				"remoteSessionID" => $this->orderFormSessionID,
				"getOrderFormItems" => 2,
				"getOrderFormProductAddons" => 1,
				"getCouponData" => 1,
				"getOrderFormParams" => 1,
				"orderFormID" => $this->orderFormID
			);

			$query = getMBAPIQuery("GetOrderForms", "", $getOptions);
			$data = dispatchMBAPI($query, 1);
			$this->orderFormSessionID =& $data["mbapi"][0]["header"][0]["remoteSessionID"][0];

			$this->orderFormData =& $data["mbapi"][0]["results"][0]["orderForms"][0]["orderForm"][0];
		}


		// Get username and password params
		foreach($this->orderFormData["orderFormParams"][0]["orderFormParam"] as $param){
			if($param["orderFormParamName"][0] == "username_min_length"){
				$ordParams["usermin"] = $param["orderFormParamValue"][0];
			}elseif($param["orderFormParamName"][0] == "username_max_length"){
				$ordParams["usermax"] = $param["orderFormParamValue"][0];
			}elseif($param["orderFormParamName"][0] == "password_min_length"){
				$ordParams["passmin"] = $param["orderFormParamValue"][0];
			}elseif($param["orderFormParamName"][0] == "password_max_length"){
				$ordParams["passmax"] = $param["orderFormParamValue"][0];
			}
		}

		return $ordParams;
	}
	
	/**
	 *	This function returns the payment options available for this order form.
	 *
	 */
	function getPaymentOptions() {

		// Get the info for this order form
		$getOptions = array(
			"remoteSessionID" => $this->orderFormSessionID,
			"getOrderFormParams" => 1,
			"getPaymentGroupData" => 1,
			"orderFormID" => $this->orderFormID
		);

		$query = getMBAPIQuery("GetOrderForms", "", $getOptions);
		$data = dispatchMBAPI($query, 1);

		$this->orderFormSessionID =& $data["mbapi"][0]["header"][0]["remoteSessionID"][0];

		$paymentInfo =& $data["mbapi"][0]["results"][0]["orderForms"][0]["orderForm"][0];

		// Get payment options and termsURL
		foreach($paymentInfo["orderFormParams"][0]["orderFormParam"] as $param){
			if($param["orderFormParamName"][0] == "payment_type_check"){
				$paymentOptions["check"] = $param["orderFormParamValue"][0];
			}elseif($param["orderFormParamName"][0] == "payment_type_invoice"){
				$paymentOptions["invoice"] = $param["orderFormParamValue"][0];
			}elseif($param["orderFormParamName"][0] == "payment_type_draft"){
				$paymentOptions["draft"] = $param["orderFormParamValue"][0];
			}elseif($param["orderFormParamName"][0] == "terms_url"){
				$paymentOptions["termsURL"] = $param["orderFormParamValue"][0];
			}
		}
		if (is_array($paymentInfo["paymentGroup"][0]["gateway"])){
			foreach($paymentInfo["paymentGroup"][0]["gateway"] as $gateway){
				if($gateway["gatewayType"][0]){
					$paymentOptions["cc"] = TRUE;
					$paymentOptions["primaryConfigGroupID"] = $gateway["configGroupID"][0];
				}else{
					$paymentOptions["secondary"][$gateway["configGroupID"][0]] = $gateway["configGroupTransKey"][0];
				}
			}
		}
		return $paymentOptions;
	}


	/**
	 *	This function returns the payment options available for this order form.
	 *
	 */
	function getExistingCustomerInfo($username, $password, $usernameOnly = FALSE) {
		global $CONF;
		
		$getOptions = array(
			"remoteSessionID" => $this->orderFormSessionID,
			"contactUsername" => $username,
			"clientActive" => 1,
			"getContactData" => 3,
			"comparisonOperator" => "="
		);
		
		if(!$usernameOnly){
			require_once mb_path("/lib-tk/include/crypt/crypt.php");
			$c =& new Crypt(getPasswordHash(PANELENCRYPTIONKEY), "blowfish", NULL);
			$encPassword = $c->encrypt($password);
			$getOptions["contactPassword"] = $encPassword;
			$getOptions["clientStatus"] = 0;
			$getOptions["getBillingData"] = 1;
		}

		$query = getMBAPIQuery("GetClients", "", $getOptions);
		$data = dispatchMBAPI($query, 1);

		$this->orderFormSessionID =& $data["mbapi"][0]["header"][0]["remoteSessionID"][0];
		
		$clientInfo =& $data["mbapi"][0]["results"][0]["clients"][0]["client"][0];

		$clientInfo = (is_array($clientInfo)) ? array_merge($clientInfo, $data["mbapi"][0]["results"][0]["clients"][0]["client"][0]["contacts"][0]["contact"][0]) : NULL;
		unset($data["mbapi"][0]["results"][0]["clients"][0]["client"][0]["contacts"]);

		return $clientInfo;
	}
	
	/**
	 *	This function checks to see if the IP or e-mail address in in the ban log.
	 *
	 */
	function checkBanning($type, $value) {
		
		$banned = FALSE;
		$getOptions = array(
			"remoteSessionID" => $this->orderFormSessionID,
			"banType" => $type,
			"banString" => $value
		);
		
		//Check to see if this is a banned email, IP, etc
		$query = getMBAPIQuery("GetBans", "", $getOptions);
		$data = dispatchMBAPI($query, 1);

		$this->orderFormSessionID =& $data["mbapi"][0]["header"][0]["remoteSessionID"][0];
		$banInfo =& $data["mbapi"][0]["results"][0]["bans"][0]["ban"][0];
		
		//If we get a return, then it is and we need to update the table and increase the count
		if (is_array($banInfo)){
				$setOptions = array(
				"remoteSessionID" => $this->orderFormSessionID,
				"banID" => $banInfo["banID"][0],
				"banCount" => ($banInfo["banCount"][0] + 1),
				"banDateLastUsed" => mktime()
			);
	
			$query = getMBAPIQuery("SetBans", "update", $setOptions);
			$setresp = dispatchMBAPI($query, 1);
			$banned = TRUE;
		}
	
		return $banned;
	} 
} 
?>
