<?php
// $Id: xml.php,v 1.62 2006/06/22 23:14:51 kbailey Exp $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+

require_once dirname(__FILE__)."/../../../lib-tk/shield.php";

/**
 * A class to parse, maintain, and malleate XML as either a string or an array.
 *
 * @author  Vic Fryzel
 * @author  Eric Pollmann <http://eric.pollmann.net/work/public_domain/>
 */
class MBXML {
	/**
	 * The array representation of the XML that this object represents.
	 */
	var $arrayData;
	
	/**
	 * The string representation of the XML that this object represents.
	 */
	var $stringData;
	
	/**
	 * The string representation of any XML errors we encounter.
	 */
	var $errorData;
	
	/**
	 * The XML parser to use throughout the life of this object.
	 */
	var $parser;
	
	/**
	 * 1 if the provided XML string was able to be parsed, or 0 if otherwise.
	 */
	var $valid;
	
	/**
	 * A constructor for an MBXML object. It stores the given input, and makes it ready for toString() and 
	 * toArray().
	 *
	 * @param  string  The string of XML, or array of structured XML to initiate this class with.
	 */
	function MBXML($input) {
		$this->arrayData = array();
		$this->stringData = $this->parseStringData = "";
		$this->valid = 0;
		if(is_array($input)) {
			$this->arrayData = $input;
			$this->valid = 1;
		} else {
			$this->stringData = $input;
			$this->parseStringData = preg_replace("/&/", "#!#ampersand#!#", $input);
		}
	}

	function handleError($parser) {
		global $CONF;
		
		$context = 40;
		
		list($err_code, $err_line, $err_col, $err_byte, $err_string) = array(
			xml_get_error_code($parser),
			xml_get_current_line_number($parser),
			 xml_get_current_column_number($parser),
			 xml_get_current_byte_index($parser),
			 xml_error_string(xml_get_error_code($parser)),
		);
		
		$pre_context = $err_byte - 1 > $context ? $context : $err_byte - 1;
		$left = strlen($this->stringData) - $err_byte;
		$post_context = $left - 1 > $context ? $context : $left - 1;
		
		$xml_err_pre = substr($this->stringData, $err_byte - $context, $context);
		$xml_err_post = substr($this->stringData, $err_byte, $context);
		
		$xml_err_pre_lines = explode("\n", $xml_err_pre);
		$xml_err_pre_line = $xml_err_pre_lines[count($xml_err_pre_lines) - 1];

		$xml_err_post_lines = explode("\n", $xml_err_post);
		$xml_err_post_line = $xml_err_post_lines[0];
		
		$xml_err_html = "<pre>" . htmlentities($xml_err_pre_line) . "<strong>|</strong><em>" . htmlentities($xml_err_post_line) . "</em></pre>";

		$this->errorData = $err_string."<br />\n".$xml_err_html;

		$CONF["debug"]->add(
			"XML Parser Failed",
			'Error ' . $err_code
				. ' at line ' . $err_line
				. ', column ' . $err_col
				. ', byte ' . $err_byte
				. ': ' . htmlentities($err_string)
				. '<br />' . $xml_err_html,
			__FILE__, __LINE__
		);
		dbg("XML Parser Failed\n" .
			'Error ' . $err_code
				. ' at line ' . $err_line
				. ', column ' . $err_col
				. ', byte ' . $err_byte
				. ': ' . htmlentities($err_string)
				. '<br />' . $xml_err_html);
	}

	/**
	 * Returns the array representation of this MBXML object.
	 *
	 * @return  array  The array representation of this MBXML object.
	 */
	function toArray() {
		$parser = xml_parser_create("UTF-8");
		xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
		xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
		if(!count($this->arrayData) && $this->parseStringData) {
			if(xml_parse_into_struct($parser, $this->parseStringData, $vals, $index) === 1
					&& xml_get_error_code($parser) === XML_ERROR_NONE)  {
				$this->valid = 1;
				$i = -1;
				$this->arrayData = $this->getChildren($vals, $i);
			} else {
				$this->handleError($parser);
			}
			xml_parser_free($parser);
		} else {
			$this->handleError($parser);
		}
		return $this->arrayData;
	}
	
	/**
	 * This private method generates and structures all of the children for the given index i in vals.
	 *
	 * @param  array  The XML structure of values to use in generation.
	 * @param  int  The index to begin at in vals.
	 * @return  array  The structured array of children for the given index.
	 */
	function getChildren($vals, &$i) {
		$children = array();
		if($i > -1 && isset($vals[$i]["value"])) {
			$children["VALUE"] = $vals[$i]["value"];
		}
		while(++$i < count($vals)) {
			$type = $vals[$i]["type"];
			if($type === "complete" || $type === "open") {
				$tag = $this->buildTag($vals[$i], $vals, $i, $type);
				$children[$vals[$i]["tag"]][] = $tag;
			} else if($type === "close") {
				break;
			} else if($type === "cdata") {
				$children["VALUE"] .= $vals[$i]["value"];
			}
		}
		return $children;
	}

	/**
	 * This private method builds out the array for a tag based upon an array that is a structure of XML.
	 *
	 * @param  thisvals  The values of the XML structure that are associated with this tag.
	 * @param  vals  All of the values existing in the XML structure.
	 * @param  i  The index to begin at in vals, passed by reference.
	 * @param  type  The step at which we are in building this tag.
	 * @return  The built tag.
	 */
	function buildTag($thisvals, $vals, &$i, $type) {
		if(isset($thisvals["attributes"])) {
			$tag["_attributes"] = $thisvals["attributes"];
		}
		
		if($type === "complete") {
			if (@!$tag["_attributes"]) {
				$tag = trim(preg_replace("/#!#ampersand#!#/i", "&", $thisvals["value"]));
			} elseif ($thisvals["value"]) {
				$tag[] = trim(preg_replace("/#!#ampersand#!#/i", "&", $thisvals["value"]));
			}
		} else {
			if(@!is_array($tag)) {
				$tag = array();
			}
			$tag = @array_merge($tag, $this->getChildren($vals, $i));
		}
		return $tag;
	}

	/**
	 * Returns the string representation of this MBXML object.
	 *
	 * @param  array  The input array to use, by default this is an empty array and $this->arrayData is used.
	 * @return  string  The string represenation of this MBXML object.
	 */
	function toString($input = array()) {
		$arr = $input;
		$retval = "";
		if(is_array($arr)) {
			if(!count($arr)) {
				$arr = $this->arrayData;
				$retval =& $this->stringData;
			}
			foreach($arr as $key => $value) {
				if(!eregi("_attributes", $key)) {
					if(is_array($value) && array_key_exists(0, $value)) {
						$c = count($value);
						for($i=0; $i<$c; $i++) {
							$retval .= "<$key";
							if(is_array($value[$i]["_attributes"])) {
								foreach($value[$i]["_attributes"] as $attrName => $attrValue) {
									$retval .= " ".$attrName."=\"".$attrValue."\"";
								}
							}
							$retval .= ">";
							if(is_array($value[$i])) {
								$retval .= "\n".$this->toString($value[$i])."\n";
							} else {
								$retval .= $this->toString($value[$i]);
							}
							$retval .= "</$key>\n";
						}
					} else {
						$retval .= "\t<$key";
						if(is_array($value["_attributes"])) {
							foreach($value["_attributes"] as $attrName => $attrValue) {
								$retval .= " ".$attrName."=\"".$attrValue."\"";
							}
						}
						$retval .= ">";
						$retval .= $this->toString($value);
						$retval .= "</$key>\n";
					}
				}
			}
		} else {
			$retval .= $arr;
		}
		return $retval;
	}
	
	/**
	 * This method returns 1 if this object was able to parse the given input XML, or 0 if otherwise. If this 
	 * object was instantiated with an array, this method will always return 1.
	 *
	 * @return  int  1 upon a successful parse of the given XML, or 0 if otherwise. Always returns 1 when this object 
	 *               has been instantiated with an array.
	 */
	function isValid() {
		return $this->valid;
	}
} // MBXML

/**
 * DEPRECATED: use correctArray
 * EXPLAINATION: collapseArrayElements will sometimes collapse things when it shouldn't and cause
 *		 you to have to test to see what level you have and correct it on every usage.
 *		 correctArray does not.  correctArray also does collaseSingularsAndPlurals on the results
 *		 which changes $array["items"][0]["item"] = array() to: $array["items"] = array();
 * This method collapses the elements in the given array, basically removing arrays that are one element
 * large and that're associated with a numeric key.
 *
 * @param  array  The array to collapse.
 * @return  array  The collapsed array.
 */
function collapseArrayElements($arr) {
	$retval = $arr;
	if(count($retval) === 1 && is_array($retval) && array_key_exists(0, $retval)) {
		$retval = $retval[0];
	}
	if(is_array($retval)) {
		foreach($retval as $key => $value) {
			$retval[$key] = collapseArrayElements($value);
		}
	}
	return $retval;
}

/**
 * Protect the arrayness of the first level from collapseArrayElements
 *
 * @param  array  to be protected
 *
 * @return  array  protected
 */
function protectArray($data) {
	if(count($data) == 1) {
		$data = array(0 => $data);
	}
	return $data;
}

/**
 * This method takes a raw mbapi result array and intelligently tries to get
 * the results of the query and return it in a collapsed manner.
 *
 * @param  array  The array of raw mbapi results.
 * @return  array  The collapsed array of results we guessed.
 */
function grabSimpleMBAPIResults($result, $checkNumResults=0, $returnFirst=0){
	if (SUPPRESS_SIMPLE_MBAPI_DEBUG<1)dbg("result at top of grab:");
	if (SUPPRESS_SIMPLE_MBAPI_DEBUG<1)dbg($result);
	if (!is_array($result)){
		return $result;
	} else {
		if (isset($result["mbapi"][0]["results"][0])){ // try to handle multiple entry points, most common first
			if (!(int)$result["mbapi"][0]["header"][0]["numResults"][0] && $checkNumResults){
				return array();
			}
		 	$result = $result["mbapi"][0]["results"][0];
		} elseif (isset($result["results"][0])){
			if (!(int)$result["header"][0]["numResults"][0] && $checkNumResults){
				return array();
			}
		 	$result = $result["results"][0];
		} elseif (isset($result[0]["results"][0])){
			if (!(int)$result[0]["header"][0]["numResults"][0] && $checkNumResults){
				return array();
			}
			$result = $result[0]["results"][0];
		} else {
			if ((int)$result["mbapi"][0]["header"][0]["errorCount"][0]){
				dbg("had an error!");
				return array();
			}
		}
		if (!is_array($result) || !count($result)){
			if (SUPPRESS_SIMPLE_MBAPI_DEBUG<1)dbg("simple results are blank array");
			return array();
		}
		reset($result);
		if ($result["paginationLinks"])unset($result["paginationLinks"]);
		if (SUPPRESS_SIMPLE_MBAPI_DEBUG<1)dbg("result after going in:");
		if (SUPPRESS_SIMPLE_MBAPI_DEBUG<1)dbg($result);

		if (count($result)==1){
			list($k, $v) = @each($result); // get first key and value off result array
			if (count($v)>1){ // if this key has a 0 object
				dbg("result we are returning ($k):");
				$x = collapseArrayElementsNonArray($v); 
				if (SUPPRESS_SIMPLE_MBAPI_DEBUG<1)dbg($x);
				return ($returnFirst)?$x[0]:$x;
			} elseif (isset($v[0]) && $v[0]==""){
				dbg("Blank results, returning blank array");
				return array();
			} else {
				$v[0] = $v[0];
				list($k2, $v) = @each($v[0]);
				dbg("result we are returning ($k / $k2):");
				$x = collapseArrayElementsNonArray($v);
				if (SUPPRESS_SIMPLE_MBAPI_DEBUG<1)dbg($x);
				return ($returnFirst)?$x[0]:$x;
			}
		} else {
		 	dbg("result count not 1");
			$x = collapseArrayElementsNonArray($result);
		 	if (SUPPRESS_SIMPLE_MBAPI_DEBUG<1)dbg($x);
			return ($returnFirst)?$x[0]:$x;
		}

	}
}

/**
 * This method collapses mbapi array returns testing for single element arrays of non-arrays.
 *
 * @param  array  The array to collapse.
 * @return  array  The collapsed array.
 */
function collapseArrayElementsNonArray($arg) {
	if (count($arg) === 1 && isset($arg[0]) && is_array($arg) && !is_array($arg[0])){
		$arg = $arg[0];
	} elseif (is_array($arg)){
		foreach ($arg as $k=>$v){
			$arg[$k] = collapseArrayElementsNonArray($v);
		}
	}
	return $arg;
}


/**
 * a method that will intelligently correct and collapse mbapi returns.
 *
 * @param	mbapi results
 * @return    intelligently collapsed array
 */
function correctArray($array, $collapsePluralsAndSingulars=1){
	global $CONF;
	$cols = array_values($CONF["dbCols"]);
	$array = collapseDBXMLCols($array, $cols);

	$array = collapseArrayElementsNonArray($array);
	if ((int)$collapsePluralsAndSingulars){
		$array = collapsePluralsAndSingulars($array);
	}
	return $array;

}

/**
 * a method that will intelligently correct and collapse mbapi returns.
 *
 * @param	mbapi results
 * @return    intelligently collapsed array
 */
function collapseDBXMLCols($array, &$cols){
	if (is_array($array)){
		foreach ($array as $k=>$v){
			if ((in_array($k, $cols) && count($v)==1 && isset($v[0]) && is_array($v)) || (is_array($v) && !is_array($v[0]) && count($v)==1 && isset($v[0]) ) ){
				$array[$k] = $v[0];
			} else {
				if (is_array($v)){
					$array[$k] = collapseDBXMLCols($v, $cols);
				}
			}
		}
	}
	return $array;

}

/**
 * a method that will change column names to xml names
 *
 * @param	array
 * @return    array with names changed.
 */
function changeColumnNamesToXML($array){
	if (is_array($array)){
		$ret = array();
		foreach ($array as $k=>$v){
			if (getDBXMLCol($k)!=""){
				$k = getDBXMLCol($k);
			}
			$ret[$k] = changeColumnNamesToXML($v);
		}
		return $ret;
	} else {
		return $array;
	}

}

/**
 * a method that will intelligently collapse arrays of form ["someItems"][0]["someItem"] = array, to ["someItems"] = array.
 *
 * @param	array to change
 * @return    intelligently collapsed array
 */
function collapsePluralsAndSingulars($array){
	// overrides array so that we can also map things like ["countries"][0]["country"]
	$overrides = array(
		"tests" => "test",
		"productChildren" => "productChild",
		"journalEntries" => "entry"
	);
	if (is_array($array)){
		foreach ($array as $k=>$v){
			if (is_array($v)){
				if (count($v)==1 && isset($v[0])){
					if (isset($overrides[$k])){
						$lookfor = $overrides[$k];
						if (SUPPRESS_SIMPLE_MBAPI_DEBUG<1)dbg("found override($k): ".$lookfor);
					} else {
						$lookfor = substr($k, 0, -1);
					}
	
					if (isset($v[0][$lookfor]) && is_array($v[0][$lookfor])){
						if (SUPPRESS_SIMPLE_MBAPI_DEBUG<1)dbg("collapsing $k [0] [$lookfor] to $k");
						$array[$k] = collapsePluralsAndSingulars($v[0][$lookfor]);
					} else {
						$array[$k] = collapsePluralsAndSingulars($v);
					}
	
				} else {
					$array[$k] = collapsePluralsAndSingulars($v);
				}
			}
		}
	}
	return $array;
}

/**
 * This method takes an array and two columns and returns an associateive array
 * meant to be put in html_options for smarty.
 *
 * @param  array  The array.
 * @param  string  the key name.
 * @param  string  the value name, or, if not in the array, the callback function name.
 * @param  array	the array of keys to ignore
 * @return  array  The keyed array.
 */
function grabHTMLOpsFromArray($arg, $key, $value, $ignoreKeys=array()) {
	$ret = array();
	$arg = (array)$arg;
	dbg("getting $key and $value from args");
	dbg("arg sent to html ops");
	dbg($arg);
	foreach ($arg as $row){
	 	if (!in_array($row[$key], $ignoreKeys)){
	 	 	if (!is_array($value)){
				if (!isset($row[$value]) && function_exists($value)){
	 	 			$ret[$row[$key]] = $value($row);
				} else {
	 	 			$ret[$row[$key]] = $row[$value];
				}
			} else {
				if (is_object($value[0])){
					$ret[$row[$key]] = call_user_func(array($value[0], $value[1]), $row);
				} else {
					dbg("not an object:");
					dbg($value);
				}
			}
		}
	}
	return $ret;
}

/**
 * function to quickly grab mbapi results from an array form of an mbapi command
 */
function grabResultsFromMBAPIArray($array){
	return grabSimpleMBAPIResults(dispatchMBAPI(toXML($array), 1));
}

/**
 * function to quickly grab mbapi results from an array form of an mbapi command
 */
function grabFirstResultFromMBAPIArray($array){
	return grabSimpleMBAPIResults(dispatchMBAPI(toXML($array), 1), 0, 1);
}

/**
 * Convert XML datestamp (Tue Apr 27 10:42:14 EDT 2004) to a Unix timestamp in seconds.
 *
 * @param string  $datestamp XML formatted date stamp
 * @return int  unix timestamp
 */
function xmlstr2unixtime($string_date) {
	$retval = 0;
	if (!empty($string_date)) {
		list($wday,$month,$day,$time,$zone,$year) = explode(' ',$string_date);
		$retval = strtotime("$wday $month $day $time $year");
	}
	return $retval;
}

/**
 * Convert XML timestamp (2004-04-26T04:06:29-04:00) to a Unix timestamp in seconds.
 *
 * @param  string  $datestamp XML formatted date stamp
 * @return  int  unix timestamp
 */
function xmldate2unixtime($timestamp) {
	$retval = 0;
	if (!empty($timestamp)) {
		list($temp_date,$temp_time) = explode('T',$timestamp);
		list($yy,$mm,$dd)      = explode('-',$temp_date);
		list($time,$zone)      = explode('-',$temp_time);
		list($hour,$min,$sec)  = explode(':',$time);
		$retval = mktime($hour,$min,$sec,$mm,$dd,$yy);
	}
	return $retval;
}

/**
 * This method will return nested XML strings if data exists.
 *
 * @return  string  The nested XML string.
 * @author  Michael Fountain
 */
function getNestedXML($array) {
	global $loopSpacer;
	$currentSpacer = $loopSpacer."  ";
	$result = "";
	if (is_array($array)) {
	 	foreach($array as $key => $value) {
	 	 	if (is_array($value)) {
	 	 	 	$result .= $currentSpacer."      <$key>\n";
				$result .= getNestedXML($value);
				$result .= $currentSpacer."      </$key>\n";
	 	 	} else {
				$result .= $currentSpacer."        <$key>$value</$key>\n";
			}
	 	}
	}
	return $result;
}

/**
 * This method will return a sanitized array
 *
 * @return array The sanitized array
 * @author Brian Smith
 */
function padMBAPIQueryArray( &$array, $prevType=false ) {
	$ret=array();
	
	if (is_array( $array ))
	foreach( $array as $k => $v ) {
		if (is_array( $v )) {
			if (is_numeric( $k )) {
				if ($prevType != "i") {
					$ret[ $k ] = padMBAPIQueryArray($v, "i");
				} elseif (count($array) <= 1) {
					$ret = padMBAPIQueryArray($v, "i");
				} else {
					$ret[ $k ] = padMBAPIQueryArray($v, "s");
				}
				continue;
			} else {
				// the key is of type string!
				$ret[ $k ] = array(padMBAPIQueryArray($v, "i"));
				continue;
			}
		} else {
			if (is_numeric( $k )) {
				if (is_numeric($k) && (int)$k === 0) {
					$ret = $v;
				} else {
					$ret[ $k ] = $v;
				}
			} else {
				switch($prevType) {
				case "i":
					$ret[ $k ] = array($v);
				break;
				case "s":
					$ret[ $k ] = $v;
				break;
				}
			}
		}
	}
	$array = $ret;
	return $ret;
}

/* Debug 
$debug = "
<testXML>
  <testChild name=\"bob\">
    <testGrandChild name=\"joe\">
      <toy>blah&?!@#$%^&*()~?,.;':\[{\]}|=+\"/</toy>
      <toy>asdf</toy>
    </testGrandChild>
    <testGrandChild name=\"andy\">
      <toy>juju</toy>
    </testGrandChild>
    <testChild>1</testChild>
  </testChild>
  <testChild>1</testChild>
  <testCan>
    <flavor>dew</flavor>
    <color>green</color>
  </testCan>
</testXML>
";
$xml =& new MBXML($debug);
echo "<pre>".htmlspecialchars($xml->toString()).print_r($xml->toArray(),1)."</pre>";
if($xml->isValid()) {
	echo "<b>VALID</b>";
} else {
	echo "<b>NOT VALID</b>";
}

echo "<pre>Correct XML:\n".htmlspecialchars($debug)."</pre>";
$xml2 =& new MBXML($xml->toArray());
echo "<pre>Our XML:\n".htmlspecialchars($xml2->toString())."</pre>";
if($xml2->isValid()) {
	echo "<b>VALID</b>";
} else {
	echo "<b>NOT VALID</b>";
}
/* */
?>
